#!/usr/bin/python
#
# Copyright (c) 2024 xuzhang3 (@xuzhang3), Fred-Sun (@Fred-Sun)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_publicipprefix_info

version_added: "2.2.0"

short_description: Get public IP prefix facts

description:
    - Get facts for a specific public IP prefix.
    - Get all facts for a specific public IP prefixes within a resource group.

options:
    name:
        description:
            - The name of the public IP prefix.
        type: str
    resource_group:
        description:
            - Limit results by resource group. Required when using name parameter.
        type: str
    tags:
        description:
            - Limit results by providing a list of tags. Format tags as 'key' or 'key:value'.
        type: list
        elements: str

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - xuzhang3 (@xuzhang3)
    - Fred-sun (@Fred-sun)
'''

EXAMPLES = '''
- name: Get facts for one Public IP Prefix
  azure_rm_publicipprefix_info:
    resource_group: myResourceGroup
    name: publicipprefix

- name: Get facts for all Public IPs within a resource groups
  azure_rm_publicipprefix_info:
    resource_group: myResourceGroup
    tags:
      - key:value
'''

RETURN = '''
publicipprefix:
    description:
        - List of public IP prefixes dicts.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID.
            returned: always
            type: str
            sample: /subscriptions/xxx---xxxxx/resourceGroups/v-xisuRG/providers/Microsoft.Network/publicIPPrefixes/pipb57dc95224
        name:
            description:
                - Name of the public IP prefix.
            returned: always
            type: str
            sample: prefix57dc95224
        type:
            description:
                - Resource type.
            returned: always
            type: str
            sample: "Microsoft.Network/publicIPPrefixes"
        location:
            description:
                - Resource location.
            returned: always
            type: str
            sample: eastus
        tags:
            description:
                - Resource tags.
            returned: always
            type: dict
            sample: {
                    "delete": "on-exit",
                    "testing": "testing"
                    }
        public_ip_address_version:
            description:
                - The public IP address version.
                - Possible values are C(IPv4) and C(IPv6).
            returned: always
            type: str
            sample: IPv4
        ip_tags:
            description:
                - The list of tags associated with the public IP prefixes.
            returned: always
            type: list
            sample: [
                    {
                        "type": "FirstPartyUsage",
                        "value": "Storage"
                    }
                    ]
        provisioning_state:
            description:
                - The provisioning state of the PublicIP Prefix resource.
                - Possible values is C(Succeeded).
            returned: always
            type: str
            sample: Succeeded
        etag:
            description:
                - A unique read-only string that changes whenever the resource is updated.
            returned: always
            type: str
            sample: "W/'1905ee13-7623-45b1-bc6b-4a12b2fb9d15'"
        sku:
            description:
                - The public IP prefix SKU.
            returned: always
            type: dict
            sample: {'name': 'standard', 'tier': 'Regional'}
        zones:
            description:
                - A list of availability zones denoting the IP allocated for the resource needs to come from.
            returned: always
            type: list
            sample: ['1', '2']
        prefix_length:
            description:
                - The Length of the Public IP Prefix.
            type: int
            returned: always
            sample: 29
        extended_location:
            description:
                - The extended location of the public ip address.
            type: str
            returned: always
            sample: 'eastus2'
        custom_ip_prefix:
            description:
                - The customIpPrefix that this prefix is associated with.
            type: dict
            returned: always
            sample: {}
        public_ip_addresses:
            description:
                - The list of all referenced PublicIPAddresses.
            type: list
            sample: []
        resource_guid:
            description:
                - The resource GUID property of the public IP prefix resource.
            type: str
            sample: "47cafa04-851d-4579-894d-74ad6afe3233"
        ip_prefix:
            description:
                - The allocated Prefix.
            type: str
            sample: 20.199.95.80/29
'''
try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

AZURE_OBJECT_CLASS = 'PublicIpPrefix'


class AzureRMPublicIPPrefixInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type='str'),
            resource_group=dict(type='str'),
            tags=dict(type='list', elements='str')
        )

        self.results = dict(
            changed=False,
        )

        self.name = None
        self.resource_group = None
        self.tags = None

        super(AzureRMPublicIPPrefixInfo, self).__init__(self.module_arg_spec,
                                                        supports_check_mode=True,
                                                        supports_tags=False,
                                                        facts_module=True)

    def exec_module(self, **kwargs):
        for key in self.module_arg_spec:
            setattr(self, key, kwargs[key])

        result = []
        if self.name is not None and self.resource_group is not None:
            result = self.get_item()
        elif self.resource_group:
            result = self.list_resource_group()
        else:
            result = self.list_all()

        raw = self.filter(result)

        self.results['publicipprefixes'] = self.format(raw)

        return self.results

    def format(self, raw):
        return [self.prefix_to_dict(item) for item in raw]

    def filter(self, response):
        return [item for item in response if self.has_tags(item.tags, self.tags)]

    def prefix_to_dict(self, prefix):
        result = dict(
            id=prefix.id,
            name=prefix.name,
            tags=prefix.tags,
            type=prefix.type,
            location=prefix.location,
            public_ip_address_version=prefix.public_ip_address_version,
            prefix_length=prefix.prefix_length,
            provisioning_state=prefix.provisioning_state,
            etag=prefix.etag,
            zones=prefix.zones,
            sku=dict(),
            ip_tags=list(),
            custom_ip_prefix=dict(),
            ip_prefix=prefix.ip_prefix,
            resource_guid=prefix.resource_guid,
            public_ip_addresses=list(),
            extended_location=prefix.extended_location
        )
        if prefix.public_ip_addresses:
            result['public_ip_addresses'] = [x.id for x in prefix.public_ip_addresses]
        if prefix.sku:
            result['sku']['name'] = prefix.sku.name
            result['sku']['tier'] = prefix.sku.tier
        if prefix.custom_ip_prefix:
            result['custom_ip_prefix']['id'] = prefix.custom_ip_prefix.id
        if prefix.ip_tags:
            result['ip_tags'] = [dict(type=x.ip_tag_type, value=x.tag) for x in prefix.ip_tags]
        return result

    def get_item(self):
        self.log('Get properties for {0}'.format(self.name))
        item = None
        try:
            item = self.network_client.public_ip_prefixes.get(self.resource_group, self.name)
        except ResourceNotFoundError:
            pass
        return [item] if item else []

    def list_resource_group(self):
        self.log('List items in resource groups')
        try:
            response = self.network_client.public_ip_prefixes.list(self.resource_group)
        except ResourceNotFoundError as exc:
            self.fail("Error listing items in resource groups {0} - {1}".format(self.resource_group, str(exc)))
        return response

    def list_all(self):
        self.log('List all items')
        try:
            response = self.network_client.public_ip_prefixes.list_all()
        except ResourceNotFoundError as exc:
            self.fail("Error listing all items - {0}".format(str(exc)))
        return response


def main():
    AzureRMPublicIPPrefixInfo()


if __name__ == '__main__':
    main()



# Copyright (c) 2019, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible_collections.community.crypto.plugins.module_utils._version import (
    LooseVersion as _LooseVersion,
)

try:
    import cryptography
    from cryptography import x509
except ImportError:
    # Error handled in the calling module.
    pass

from ansible_collections.community.crypto.plugins.module_utils._crypto._obj2txt import (
    obj2txt,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    HAS_CRYPTOGRAPHY,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    CRYPTOGRAPHY_TIMEZONE,
    cryptography_decode_name,
)

if t.TYPE_CHECKING:
    import datetime  # pragma: no cover


# TODO: once cryptography has a _utc variant of InvalidityDate.invalidity_date, set this
#       to True and adjust get_invalidity_date() accordingly.
#       (https://github.com/pyca/cryptography/issues/10818)
CRYPTOGRAPHY_TIMEZONE_INVALIDITY_DATE = False  # pylint: disable=invalid-name
if HAS_CRYPTOGRAPHY:
    CRYPTOGRAPHY_TIMEZONE_INVALIDITY_DATE = _LooseVersion(
        cryptography.__version__
    ) >= _LooseVersion("43.0.0")

TIMESTAMP_FORMAT = "%Y%m%d%H%M%SZ"


if HAS_CRYPTOGRAPHY:
    REVOCATION_REASON_MAP = {
        "unspecified": x509.ReasonFlags.unspecified,
        "key_compromise": x509.ReasonFlags.key_compromise,
        "ca_compromise": x509.ReasonFlags.ca_compromise,
        "affiliation_changed": x509.ReasonFlags.affiliation_changed,
        "superseded": x509.ReasonFlags.superseded,
        "cessation_of_operation": x509.ReasonFlags.cessation_of_operation,
        "certificate_hold": x509.ReasonFlags.certificate_hold,
        "privilege_withdrawn": x509.ReasonFlags.privilege_withdrawn,
        "aa_compromise": x509.ReasonFlags.aa_compromise,
        "remove_from_crl": x509.ReasonFlags.remove_from_crl,
    }
    REVOCATION_REASON_MAP_INVERSE = {}
    for k, v in REVOCATION_REASON_MAP.items():
        REVOCATION_REASON_MAP_INVERSE[v] = k

else:
    REVOCATION_REASON_MAP = {}
    REVOCATION_REASON_MAP_INVERSE = {}


def cryptography_decode_revoked_certificate(
    cert: x509.RevokedCertificate,
) -> dict[str, t.Any]:
    result = {
        "serial_number": cert.serial_number,
        "revocation_date": get_revocation_date(cert),
        "issuer": None,
        "issuer_critical": False,
        "reason": None,
        "reason_critical": False,
        "invalidity_date": None,
        "invalidity_date_critical": False,
    }
    try:
        ext_ci = cert.extensions.get_extension_for_class(x509.CertificateIssuer)
        result["issuer"] = list(ext_ci.value)
        result["issuer_critical"] = ext_ci.critical
    except x509.ExtensionNotFound:
        pass
    try:
        ext_cr = cert.extensions.get_extension_for_class(x509.CRLReason)
        result["reason"] = ext_cr.value.reason
        result["reason_critical"] = ext_cr.critical
    except x509.ExtensionNotFound:
        pass
    try:
        ext_id = cert.extensions.get_extension_for_class(x509.InvalidityDate)
        result["invalidity_date"] = get_invalidity_date(ext_id.value)
        result["invalidity_date_critical"] = ext_id.critical
    except x509.ExtensionNotFound:
        pass
    return result


def cryptography_dump_revoked(
    entry: dict[str, t.Any],
    *,
    idn_rewrite: t.Literal["ignore", "idna", "unicode"] = "ignore",
) -> dict[str, t.Any]:
    return {
        "serial_number": entry["serial_number"],
        "revocation_date": entry["revocation_date"].strftime(TIMESTAMP_FORMAT),
        "issuer": (
            [
                cryptography_decode_name(issuer, idn_rewrite=idn_rewrite)
                for issuer in entry["issuer"]
            ]
            if entry["issuer"] is not None
            else None
        ),
        "issuer_critical": entry["issuer_critical"],
        "reason": (
            REVOCATION_REASON_MAP_INVERSE.get(entry["reason"])
            if entry["reason"] is not None
            else None
        ),
        "reason_critical": entry["reason_critical"],
        "invalidity_date": (
            entry["invalidity_date"].strftime(TIMESTAMP_FORMAT)
            if entry["invalidity_date"] is not None
            else None
        ),
        "invalidity_date_critical": entry["invalidity_date_critical"],
    }


def cryptography_get_signature_algorithm_oid_from_crl(
    crl: x509.CertificateRevocationList,
) -> x509.oid.ObjectIdentifier:
    try:
        return crl.signature_algorithm_oid
    except AttributeError:
        # Older cryptography versions do not have signature_algorithm_oid yet
        dotted = obj2txt(
            crl._backend._lib,  # type: ignore[attr-defined]  # pylint: disable=protected-access
            crl._backend._ffi,  # type: ignore[attr-defined]  # pylint: disable=protected-access
            crl._x509_crl.sig_alg.algorithm,  # type: ignore[attr-defined]  # pylint: disable=protected-access
        )
        return x509.oid.ObjectIdentifier(dotted)


def get_next_update(obj: x509.CertificateRevocationList) -> datetime.datetime | None:
    if CRYPTOGRAPHY_TIMEZONE:
        return obj.next_update_utc
    return obj.next_update


def get_last_update(obj: x509.CertificateRevocationList) -> datetime.datetime:
    if CRYPTOGRAPHY_TIMEZONE:
        return obj.last_update_utc
    return obj.last_update


def get_revocation_date(obj: x509.RevokedCertificate) -> datetime.datetime:
    if CRYPTOGRAPHY_TIMEZONE:
        return obj.revocation_date_utc
    return obj.revocation_date


def get_invalidity_date(obj: x509.InvalidityDate) -> datetime.datetime:
    if CRYPTOGRAPHY_TIMEZONE_INVALIDITY_DATE:
        return obj.invalidity_date_utc
    return obj.invalidity_date


def set_next_update(
    builder: x509.CertificateRevocationListBuilder, *, value: datetime.datetime
) -> x509.CertificateRevocationListBuilder:
    return builder.next_update(value)


def set_last_update(
    builder: x509.CertificateRevocationListBuilder, *, value: datetime.datetime
) -> x509.CertificateRevocationListBuilder:
    return builder.last_update(value)


def set_revocation_date(
    builder: x509.RevokedCertificateBuilder, *, value: datetime.datetime
) -> x509.RevokedCertificateBuilder:
    return builder.revocation_date(value)


__all__ = (
    "REVOCATION_REASON_MAP",
    "REVOCATION_REASON_MAP_INVERSE",
    "cryptography_decode_revoked_certificate",
    "cryptography_dump_revoked",
    "cryptography_get_signature_algorithm_oid_from_crl",
    "get_invalidity_date",
    "get_last_update",
    "get_next_update",
    "get_revocation_date",
    "set_last_update",
    "set_next_update",
    "set_revocation_date",
)



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_bgp_address_family
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_bgp_address_family
short_description: BGP Address Family resource module.
description:
- This module manages BGP Address Family configuration on devices running Cisco NX-OS.
version_added: 2.0.0
notes:
- Tested against NX-OS 9.3.6.
- Unsupported for Cisco MDS
- For managing BGP neighbor address family configurations please use
  the M(cisco.nxos.nxos_bgp_neighbor_address_family) module.
- This module works with connection C(network_cli) and C(httpapi).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section '^router bgp').
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A list of BGP process configuration.
    type: dict
    suboptions:
      as_number:
        description: Autonomous System Number of the router.
        type: str
      address_family:
        description: Address Family related configurations.
        type: list
        elements: dict
        suboptions:
          afi:
            description: Address Family indicator.
            type: str
            choices: ["ipv4", "ipv6", "link-state", "vpnv4", "vpnv6", "l2vpn"]
            required: true
          safi:
            description: Sub Address Family indicator.
            type: str
            choices: ["unicast", "multicast", "mvpn", "evpn"]
          additional_paths:
            description: Additional paths configuration.
            type: dict
            suboptions:
              install_backup:
                description: Install backup path.
                type: bool
              receive:
                description: Additional paths Receive capability.
                type: bool
              selection:
                description: Additional paths selection
                type: dict
                suboptions:
                  route_map:
                    description: Route-map for additional paths selection
                    type: str
              send:
                description: Additional paths Send capability
                type: bool
          advertise_pip:
            description: Advertise physical ip for type-5 route.
            type: bool
          advertise_l2vpn_evpn:
            description: Enable advertising EVPN routes.
            type: bool
          advertise_system_mac:
            description: Advertise extra EVPN RT-2 with system MAC.
            type: bool
          allow_vni_in_ethertag:
            description: Allow VNI in Ethernet Tag field in EVPN route.
            type: bool
          aggregate_address:
            description: Configure BGP aggregate prefixes
            type: list
            elements: dict
            suboptions:
              prefix:
                description: Aggregate prefix.
                type: str
              advertise_map:
                description: Select attribute information from specific routes.
                type: str
              as_set:
                description: Generate AS-SET information.
                type: bool
              attribute_map:
                description: Set attribute information of aggregate.
                type: str
              summary_only:
                description: Do not advertise more specifics.
                type: bool
              suppress_map:
                description: Conditionally filter more specific routes.
                type: str
          client_to_client:
            description: Configure client-to-client route reflection.
            type: dict
            suboptions:
              no_reflection:
                description: Reflection of routes permitted.
                type: bool
          dampen_igp_metric:
            description: Dampen IGP metric-related changes.
            type: int
          dampening:
            description: Configure route flap dampening.
            type: dict
            suboptions:
              set:
                description: Set route flap dampening.
                type: bool
              decay_half_life:
                description: Decay half life.
                type: int
              start_reuse_route:
                description: Value to start reusing a route.
                type: int
              start_suppress_route:
                description: Value to start suppressing a route.
                type: int
              max_suppress_time:
                description: Maximum suppress time for stable route.
                type: int
              route_map:
                description: Apply route-map to specify dampening criteria.
                type: str
          default_information:
            description: Control distribution of default information.
            type: dict
            suboptions:
              originate:
                description: Distribute a default route.
                type: bool
          default_metric:
            description: Set metric of redistributed routes.
            type: int
          distance:
            description: Configure administrative distance.
            type: dict
            suboptions:
              ebgp_routes:
                description: Distance for EBGP routes.
                type: int
              ibgp_routes:
                description: Distance for IBGP routes.
                type: int
              local_routes:
                description: Distance for local routes.
                type: int
          export_gateway_ip:
            description: Export Gateway IP to Type-5 EVPN routes for VRF
            type: bool
          inject_map:
            description: Routemap which specifies prefixes to inject.
            type: list
            elements: dict
            suboptions:
              route_map:
                description: Route-map name.
                type: str
              exist_map:
                description: Routemap which specifies exist condition.
                type: str
              copy_attributes:
                description: Copy attributes from aggregate.
                type: bool
          maximum_paths:
            description: Forward packets over multipath paths.
            type: dict
            suboptions:
              parallel_paths:
                description: Number of parallel paths.
                type: int
              ibgp:
                description: Configure multipath for IBGP paths.
                type: dict
                suboptions:
                  parallel_paths:
                    description:  Number of parallel paths.
                    type: int
              eibgp:
                description: Configure multipath for both EBGP and IBGP paths.
                type: dict
                suboptions:
                  parallel_paths:
                    description:  Number of parallel paths.
                    type: int
              local:
                description: Configure multipath for local paths.
                type: dict
                suboptions:
                  parallel_paths:
                    description:  Number of parallel paths.
                    type: int
              mixed:
                description: Configure multipath for local and remote paths.
                type: dict
                suboptions:
                  parallel_paths:
                    description:  Number of parallel paths.
                    type: int
          networks:
            description: Configure an IP prefix to advertise.
            type: list
            elements: dict
            suboptions:
              prefix:
                description: IP prefix in CIDR format.
                type: str
              route_map:
                description: Route-map name.
                type: str
          nexthop:
            description: Nexthop tracking.
            type: dict
            suboptions:
              route_map:
                description: Route-map name.
                type: str
              trigger_delay:
                description: Set the delay to trigger nexthop tracking.
                type: dict
                suboptions:
                  critical_delay:
                    description:
                    - Nexthop changes affecting reachability.
                    - Delay value (miliseconds).
                    type: int
                  non_critical_delay:
                    description:
                    - Other nexthop changes.
                    - Delay value (miliseconds).
                    type: int
          redistribute:
            description: Configure redistribution.
            type: list
            elements: dict
            suboptions:
              protocol:
                description:
                - The name of the protocol.
                type: str
                choices: ["am", "direct", "eigrp", "isis", "lisp", "ospf", "ospfv3", "rip", "static", "hmm"]
                required: true
              id:
                description:
                - The identifier for the protocol specified.
                type: str
              route_map:
                description:
                - The route map policy to constrain redistribution.
                type: str
                required: true
          retain:
            description: Retain the routes based on Target VPN Extended Communities.
            type: dict
            suboptions:
              route_target:
                description: Specify Target VPN Extended Communities
                type: dict
                suboptions:
                  retain_all:
                    description: All the routes regardless of Target-VPN community
                    type: bool
                  route_map:
                    description: Apply route-map to filter routes.
                    type: str
          suppress_inactive:
            description: Advertise only active routes to peers.
            type: bool
          table_map:
            description:
            - Policy for filtering/modifying OSPF routes before sending them to RIB.
            type: dict
            suboptions:
              name:
                description:
                - The Route Map name.
                type: str
                required: true
              filter:
                description:
                - Block the OSPF routes from being sent to RIB.
                type: bool
          timers:
            description: Configure bgp related timers.
            type: dict
            suboptions:
              bestpath_defer:
                description: Configure bestpath defer timer value for batch prefix processing.
                type: dict
                suboptions:
                  defer_time:
                    description: Bestpath defer time (mseconds).
                    type: int
                  maximum_defer_time:
                    description: Maximum bestpath defer time (mseconds).
                    type: int
          wait_igp_convergence:
            description: Delay initial bestpath until redistributed IGPs have converged.
            type: bool
          vrf:
            description: Virtual Router Context.
            type: str
  state:
    description:
    - The state the configuration should be left in.
    - State I(deleted) only removes BGP attributes that this modules
      manages and does not negate the BGP process completely.
    - Refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged
"""
EXAMPLES = """
# Using merged

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# Nexus9000v#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_bgp_address_family:
    config:
      as_number: 65536
      address_family:
        - afi: ipv4
          safi: multicast
          networks:
            - prefix: 192.0.2.32/27
            - prefix: 192.0.2.64/27
              route_map: rmap1
          nexthop:
            route_map: rmap2
            trigger_delay:
              critical_delay: 120
              non_critical_delay: 180
        - afi: ipv4
          safi: unicast
          vrf: site-1
          default_information:
            originate: true
          aggregate_address:
            - prefix: 203.0.113.0/24
              as_set: true
              summary_only: true
        - afi: ipv6
          safi: multicast
          vrf: site-1
          redistribute:
            - protocol: ospfv3
              id: 100
              route_map: rmap-ospf-1
            - protocol: eigrp
              id: 101
              route_map: rmap-eigrp-1

# Task output:
# ------------
#  before: {}
#
#  commands:
#  - router bgp 65536
#  - address-family ipv4 multicast
#  - nexthop route-map rmap2
#  - nexthop trigger-delay critical 120 non-critical 180
#  - network 192.0.2.32/27
#  - network 192.0.2.64/27 route-map rmap1
#  - vrf site-1
#  - address-family ipv4 unicast
#  - default-information originate
#  - aggregate-address 203.0.113.0/24 as-set summary-only
#  - address-family ipv6 multicast
#  - redistribute ospfv3 100 route-map rmap-ospf-1
#  - redistribute eigrp 101 route-map rmap-eigrp-1
#
#  after:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#     network 192.0.2.64/27 route-map rmap1
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1
#

# Using replaced

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#     network 192.0.2.64/27 route-map rmap1
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1

- name: Replace configuration of specified AFs
  cisco.nxos.nxos_bgp_address_family:
    config:
      as_number: 65536
      address_family:
        - afi: ipv4
          safi: multicast
          networks:
            - prefix: 192.0.2.64/27
              route_map: rmap1
          nexthop:
            route_map: rmap2
            trigger_delay:
              critical_delay: 120
              non_critical_delay: 180
          aggregate_address:
            - prefix: 203.0.113.0/24
              as_set: true
              summary_only: true
        - afi: ipv4
          safi: unicast
          vrf: site-1
    state: replaced

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1
#
#  commands:
#  - router bgp 65536
#  - address-family ipv4 multicast
#  - no network 192.0.2.32/27
#  - aggregate-address 203.0.113.0/24 as-set summary-only
#  - vrf site-1
#  - address-family ipv4 unicast
#  - no default-information originate
#  - no aggregate-address 203.0.113.0/24 as-set summary-only
#
#  after:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - protocol: ospfv3
#            id: "100"
#            route_map: rmap-ospf-1
#          - protocol: eigrp
#            id: "101"
#            route_map: rmap-eigrp-1

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.64/27 route-map rmap1
#     aggregate-address 203.0.113.0/24 as-set summary-only
#   vrf site-1
#     address-family ipv4 unicast
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1

# Using overridden

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#     network 192.0.2.64/27 route-map rmap1
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1

- name: Override all BGP AF configuration with provided configuration
  cisco.nxos.nxos_bgp_address_family: &overridden
    config:
      as_number: 65536
      address_family:
        - afi: ipv4
          safi: multicast
          networks:
            - prefix: 192.0.2.64/27
              route_map: rmap1
          aggregate_address:
            - prefix: 203.0.113.0/24
              as_set: true
              summary_only: true
        - afi: ipv4
          safi: unicast
          vrf: site-1
    state: overridden

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1
#
#  commands:
#  - router bgp 65536
#  - vrf site-1
#  - no address-family ipv6 multicast
#  - exit
#  - address-family ipv4 multicast
#  - no nexthop route-map rmap2
#  - no nexthop trigger-delay critical 120 non-critical 180
#  - aggregate-address 203.0.113.0/24 as-set summary-only
#  - no network 192.0.2.32/27
#  - vrf site-1
#  - address-family ipv4 unicast
#  - no default-information originate
#  - no aggregate-address 203.0.113.0/24 as-set summary-only
#
#  after:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1

#
# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     network 192.0.2.64/27 route-map rmap1
#     aggregate-address 203.0.113.0/24 as-set summary-only
#   vrf site-1
#     address-family ipv4 unicast
#

# Using deleted to remove specified AFs

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#     network 192.0.2.64/27 route-map rmap1
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1

- name: Delete specified BGP AFs
  cisco.nxos.nxos_bgp_address_family:
    config:
      as_number: 65536
      address_family:
        - afi: ipv4
          safi: multicast
        - vrf: site-1
          afi: ipv6
          safi: multicast
    state: deleted

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1
#
#  commands:
#  - router bgp 65563
#  - no address-family ipv4 multicast
#  - vrf site-1
#  - no address-family ipv6 multicast
#
#  after:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only

# Using deleted to remove all BGP AFs

# Before state:
# -------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#     network 192.0.2.64/27 route-map rmap1
#   vrf site-1
#     address-family ipv4 unicast
#       default-information originate
#       aggregate-address 203.0.113.0/24 as-set summary-only
#     address-family ipv6 multicast
#       redistribute ospfv3 100 route-map rmap-ospf-1
#       redistribute eigrp 101 route-map rmap-eigrp-1

- name: Delete all BGP AFs
  cisco.nxos.nxos_bgp_address_family:
    state: deleted

# Task output:
# ------------
#  before:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1
#
#  commands:
#  - router bgp 65563
#  - no address-family ipv4 multicast
#  - vrf site-1
#  - no address-family ipv4 unicast
#  - no address-family ipv6 multicast
#
#  after:
#    as_number: "65536"

# After state:
# ------------
# Nexus9000v# show running-config | section "^router bgp"
# router bgp 65536
# Nexus9000v#

# Using rendered

- name: Render platform specific configuration lines with state rendered (without connecting to the device)
  cisco.nxos.nxos_bgp_address_family:
    config:
      as_number: 65536
      address_family:
        - afi: ipv4
          safi: multicast
          networks:
            - prefix: 192.0.2.32/27
            - prefix: 192.0.2.64/27
              route_map: rmap1
          nexthop:
            route_map: rmap2
            trigger_delay:
              critical_delay: 120
              non_critical_delay: 180
        - afi: ipv4
          safi: unicast
          vrf: site-1
          default_information:
            originate: true
          aggregate_address:
            - prefix: 203.0.113.0/24
              as_set: true
              summary_only: true
        - afi: ipv6
          safi: multicast
          vrf: site-1
          redistribute:
            - protocol: ospfv3
              id: 100
              route_map: rmap-ospf-1
            - protocol: eigrp
              id: 101
              route_map: rmap-eigrp-1
    state: rendered

# Task Output:
# ------------
# rendered:
# - router bgp 65536
# - address-family ipv4 multicast
# - nexthop route-map rmap2
# - nexthop trigger-delay critical 120 non-critical 180
# - network 192.0.2.32/27
# - network 192.0.2.64/27 route-map rmap1
# - vrf site-1
# - address-family ipv4 unicast
# - default-information originate
# - aggregate-address 203.0.113.0/24 as-set summary-only
# - address-family ipv6 multicast
# - redistribute ospfv3 100 route-map rmap-ospf-1
# - redistribute eigrp 101 route-map rmap-eigrp-1

# Using parsed

# parsed.cfg
# ------------
# router bgp 65536
#   address-family ipv4 multicast
#     nexthop route-map rmap2
#     nexthop trigger-delay critical 120 non-critical 180
#     network 192.0.2.32/27
#    network 192.0.2.64/27 route-map rmap1
#  vrf site-1
#    address-family ipv4 unicast
#      default-information originate
#      aggregate-address 203.0.113.0/24 as-set summary-only
#    address-family ipv6 multicast
#      redistribute ospfv3 100 route-map rmap-ospf-1
#      redistribute eigrp 101 route-map rmap-eigrp-1

- name: Parse externally provided BGP AF config
  cisco.nxos.nxos_bgp_address_family:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output:
# ------------
#  parsed:
#    as_number: "65536"
#    address_family:
#      - afi: ipv4
#        safi: multicast
#        networks:
#          - prefix: 192.0.2.32/27
#          - prefix: 192.0.2.64/27
#            route_map: rmap1
#        nexthop:
#          route_map: rmap2
#          trigger_delay:
#            critical_delay: 120
#            non_critical_delay: 180
#      - afi: ipv4
#        safi: unicast
#        vrf: site-1
#        default_information:
#          originate: true
#        aggregate_address:
#          - prefix: 203.0.113.0/24
#            as_set: true
#            summary_only: true
#      - afi: ipv6
#        safi: multicast
#        vrf: site-1
#        redistribute:
#          - id: "100"
#            protocol: ospfv3
#            route_map: rmap-ospf-1
#          - id: "101"
#            protocol: eigrp
#            route_map: rmap-eigrp-1
"""
RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - router bgp 65536
    - address-family ipv4 multicast
    - nexthop route-map rmap2
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - router bgp 65536
    - address-family ipv4 multicast
    - nexthop route-map rmap2
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.bgp_address_family.bgp_address_family import (
    Bgp_address_familyArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.bgp_address_family.bgp_address_family import (
    Bgp_address_family,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Bgp_address_familyArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Bgp_address_family(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2021, Cisco Systems
# GNU General Public License v3.0+ (see LICENSE or
# https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: networks_appliance_firewall_cellular_firewall_rules
short_description: Resource module for networks _appliance _firewall _cellular _firewall
  _rules
description:
  - Manage operation update of the resource networks _appliance _firewall _cellular
    _firewall _rules.
  - Update the cellular firewall rules of an MX network.
version_added: '1.0.0'
extends_documentation_fragment:
  - cisco.meraki.module
author: Francisco Munoz (@fmunoz)
options:
  networkId:
    description: NetworkId path parameter. Network ID.
    type: str
  rules:
    description: An ordered array of the firewall rules (not including the default
      rule).
    elements: dict
    suboptions:
      comment:
        description: Description of the rule (optional).
        type: str
      destCidr:
        description: Comma-separated list of destination IP address(es) (in IP or
          CIDR notation), fully-qualified domain names (FQDN) or 'any'.
        type: str
      destPort:
        description: Comma-separated list of destination port(s) (integer in the range
          1-65535), or 'any'.
        type: str
      policy:
        description: '''allow'' or ''deny'' traffic specified by this rule.'
        type: str
      protocol:
        description: The type of protocol (must be 'tcp', 'udp', 'icmp', 'icmp6' or
          'any').
        type: str
      srcCidr:
        description: Comma-separated list of source IP address(es) (in IP or CIDR
          notation), or 'any' (note FQDN not supported for source addresses).
        type: str
      srcPort:
        description: Comma-separated list of source port(s) (integer in the range
          1-65535), or 'any'.
        type: str
      syslogEnabled:
        description: Log this rule to syslog (true or false, boolean value) - only
          applicable if a syslog has been configured (optional).
        type: bool
    type: list
requirements:
  - meraki >= 2.4.9
  - python >= 3.5
seealso:
  - name: Cisco Meraki documentation for appliance updateNetworkApplianceFirewallCellularFirewallRules
    description: Complete reference of the updateNetworkApplianceFirewallCellularFirewallRules
      API.
    link: https://developer.cisco.com/meraki/api-v1/#!update-network-appliance-firewall-cellular-firewall-rules
notes:
  - SDK Method used are
    appliance.Appliance.update_network_appliance_firewall_cellular_firewall_rules,
  - Paths used are
    put /networks/{networkId}/appliance/firewall/cellularFirewallRules,
"""

EXAMPLES = r"""
- name: Update all
  cisco.meraki.networks_appliance_firewall_cellular_firewall_rules:
    meraki_api_key: "{{ meraki_api_key }}"
    meraki_base_url: "{{ meraki_base_url }}"
    meraki_single_request_timeout: "{{ meraki_single_request_timeout }}"
    meraki_certificate_path: "{{ meraki_certificate_path }}"
    meraki_requests_proxy: "{{ meraki_requests_proxy }}"
    meraki_wait_on_rate_limit: "{{ meraki_wait_on_rate_limit }}"
    meraki_nginx_429_retry_wait_time: "{{ meraki_nginx_429_retry_wait_time }}"
    meraki_action_batch_retry_wait_time: "{{ meraki_action_batch_retry_wait_time }}"
    meraki_retry_4xx_error: "{{ meraki_retry_4xx_error }}"
    meraki_retry_4xx_error_wait_time: "{{ meraki_retry_4xx_error_wait_time }}"
    meraki_maximum_retries: "{{ meraki_maximum_retries }}"
    meraki_output_log: "{{ meraki_output_log }}"
    meraki_log_file_prefix: "{{ meraki_log_file_prefix }}"
    meraki_log_path: "{{ meraki_log_path }}"
    meraki_print_console: "{{ meraki_print_console }}"
    meraki_suppress_logging: "{{ meraki_suppress_logging }}"
    meraki_simulate: "{{ meraki_simulate }}"
    meraki_be_geo_id: "{{ meraki_be_geo_id }}"
    meraki_caller: "{{ meraki_caller }}"
    meraki_use_iterator_for_get_pages: "{{ meraki_use_iterator_for_get_pages }}"
    meraki_inherit_logging_config: "{{ meraki_inherit_logging_config }}"
    state: present
    networkId: string
    rules:
      - comment: Allow TCP traffic to subnet with HTTP servers.
        destCidr: 192.168.1.0/24
        destPort: '443'
        policy: allow
        protocol: tcp
        srcCidr: Any
        srcPort: Any
        syslogEnabled: false
"""
RETURN = r"""
meraki_response:
  description: A dictionary or list with the response returned by the Cisco Meraki Python SDK
  returned: always
  type: dict
  sample: >
    {}
"""



#!/usr/bin/python
# Copyright (c) 2019, Patrick Pichler <ppichler+ansible@mgit.at>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: openssl_signature
version_added: 1.1.0
short_description: Sign data with openssl
description:
  - This module allows one to sign data using a private key.
  - The module uses the cryptography Python library.
author:
  - Patrick Pichler (@aveexy)
  - Markus Teufelberger (@MarkusTeufelberger)
extends_documentation_fragment:
  - community.crypto._attributes
  - community.crypto._cryptography_dep.minimum
attributes:
  check_mode:
    support: full
    details:
      - This action does not modify state.
  diff_mode:
    support: none
  idempotent:
    support: partial
    details:
      - Signature algorithms are generally not deterministic. Thus the generated signature
        can change from one invocation to the next.
options:
  privatekey_path:
    description:
      - The path to the private key to use when signing.
      - Either O(privatekey_path) or O(privatekey_content) must be specified, but not both.
    type: path
  privatekey_content:
    description:
      - The content of the private key to use when signing the certificate signing request.
      - Either O(privatekey_path) or O(privatekey_content) must be specified, but not both.
    type: str
  privatekey_passphrase:
    description:
      - The passphrase for the private key.
      - This is required if the private key is password protected.
    type: str
  path:
    description:
      - The file to sign.
      - This file will only be read and not modified.
    type: path
    required: true
  select_crypto_backend:
    description:
      - Determines which crypto backend to use.
      - The default choice is V(auto), which tries to use C(cryptography) if available.
      - If set to V(cryptography), will try to use the L(cryptography,https://cryptography.io/) library.
      - Note that with community.crypto 3.0.0, all values behave the same.
        This option will be deprecated in a later version.
        We recommend to not set it explicitly.
    type: str
    default: auto
    choices: [auto, cryptography]
seealso:
  - module: community.crypto.openssl_signature_info
  - module: community.crypto.openssl_privatekey
"""

EXAMPLES = r"""
---
- name: Sign example file
  community.crypto.openssl_signature:
    privatekey_path: private.key
    path: /tmp/example_file
  register: sig

- name: Verify signature of example file
  community.crypto.openssl_signature_info:
    certificate_path: cert.pem
    path: /tmp/example_file
    signature: "{{ sig.signature }}"
  register: verify

- name: Make sure the signature is valid
  ansible.builtin.assert:
    that:
      - verify.valid
"""

RETURN = r"""
signature:
  description: Base64 encoded signature.
  returned: success
  type: str
"""

import base64
import os
import typing as t

from ansible_collections.community.crypto.plugins.module_utils._cryptography_dep import (
    COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION,
    assert_required_cryptography_version,
)
from ansible_collections.community.crypto.plugins.module_utils._version import (
    LooseVersion,
)

MINIMAL_CRYPTOGRAPHY_VERSION = COLLECTION_MINIMUM_CRYPTOGRAPHY_VERSION

try:
    import cryptography
    import cryptography.hazmat.primitives.asymmetric.padding
    import cryptography.hazmat.primitives.hashes

except ImportError:
    CRYPTOGRAPHY_VERSION = LooseVersion("0.0")  # pylint: disable=invalid-name
else:
    # pylint: disable-next=invalid-name
    CRYPTOGRAPHY_VERSION = LooseVersion(cryptography.__version__)

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLObjectError,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.support import (
    OpenSSLObject,
    load_privatekey,
)


class SignatureBase(OpenSSLObject):
    def __init__(self, module: AnsibleModule) -> None:
        super().__init__(
            path=module.params["path"],
            state="present",
            force=False,
            check_mode=module.check_mode,
        )

        self.module = module
        self.privatekey_path: str | None = module.params["privatekey_path"]
        privatekey_content: str | None = module.params["privatekey_content"]
        if privatekey_content is not None:
            self.privatekey_content: bytes | None = privatekey_content.encode("utf-8")
        else:
            self.privatekey_content = None
        self.privatekey_passphrase: str | None = module.params["privatekey_passphrase"]

    def generate(self, module: AnsibleModule) -> None:
        # Empty method because OpenSSLObject wants this
        pass

    def dump(self) -> dict[str, t.Any]:
        # Empty method because OpenSSLObject wants this
        return {}


# Implementation with using cryptography
class SignatureCryptography(SignatureBase):
    def __init__(self, module: AnsibleModule) -> None:
        super().__init__(module)

    def run(self) -> dict[str, t.Any]:
        _padding = cryptography.hazmat.primitives.asymmetric.padding.PKCS1v15()
        _hash = cryptography.hazmat.primitives.hashes.SHA256()

        result: dict[str, t.Any] = {}

        try:
            with open(self.path, "rb") as f:
                _in = f.read()

            private_key = load_privatekey(
                path=self.privatekey_path,
                content=self.privatekey_content,
                passphrase=self.privatekey_passphrase,
            )

            signature = None

            if isinstance(
                private_key,
                cryptography.hazmat.primitives.asymmetric.dsa.DSAPrivateKey,
            ):
                signature = private_key.sign(_in, _hash)

            elif isinstance(
                private_key,
                cryptography.hazmat.primitives.asymmetric.ec.EllipticCurvePrivateKey,
            ):
                signature = private_key.sign(
                    _in, cryptography.hazmat.primitives.asymmetric.ec.ECDSA(_hash)
                )

            elif isinstance(
                private_key,
                (
                    cryptography.hazmat.primitives.asymmetric.ed25519.Ed25519PrivateKey,
                    cryptography.hazmat.primitives.asymmetric.ed448.Ed448PrivateKey,
                ),
            ):
                signature = private_key.sign(_in)

            elif isinstance(
                private_key,
                cryptography.hazmat.primitives.asymmetric.rsa.RSAPrivateKey,
            ):
                signature = private_key.sign(_in, _padding, _hash)

            if signature is None:
                self.module.fail_json(
                    msg=f"Unsupported key type. Your cryptography version is {CRYPTOGRAPHY_VERSION}"
                )

            result["signature"] = base64.b64encode(signature)
            return result

        except Exception as e:
            raise OpenSSLObjectError(e) from e


def main() -> t.NoReturn:
    module = AnsibleModule(
        argument_spec={
            "privatekey_path": {"type": "path"},
            "privatekey_content": {"type": "str", "no_log": True},
            "privatekey_passphrase": {"type": "str", "no_log": True},
            "path": {"type": "path", "required": True},
            "select_crypto_backend": {
                "type": "str",
                "choices": ["auto", "cryptography"],
                "default": "auto",
            },
        },
        mutually_exclusive=(["privatekey_path", "privatekey_content"],),
        required_one_of=(["privatekey_path", "privatekey_content"],),
        supports_check_mode=True,
    )

    if not os.path.isfile(module.params["path"]):
        module.fail_json(
            name=module.params["path"],
            msg=f"The file {module.params['path']} does not exist",
        )

    assert_required_cryptography_version(
        module, minimum_cryptography_version=MINIMAL_CRYPTOGRAPHY_VERSION
    )

    try:
        _sign = SignatureCryptography(module)

        result = _sign.run()

        module.exit_json(**result)
    except OpenSSLObjectError as exc:
        module.fail_json(msg=str(exc))


if __name__ == "__main__":
    main()



# Copyright (c) 2021 Felix Fontein
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import typing as t

from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.dns.plugins.module_utils._conversion.base import (
    DNSConversionError,
)

_DECIMAL_DIGITS = b"0123456789"

_STATE_OUTSIDE = 0
_STATE_QUOTED_STRING = 1
_STATE_UNQUOTED_STRING = 3


def _parse_quoted(value: bytes, index: int, use_octal: bool) -> tuple[bytes, int]:
    if index == len(value):
        raise DNSConversionError("Unexpected backslash at end of string")
    letter = value[index : index + 1]
    index += 1
    if letter in (b"\\", b'"'):
        return letter, index
    # This must be a decimal sequence
    v2 = _DECIMAL_DIGITS.find(letter)
    if v2 < 0 or (use_octal and v2 >= 8):
        # It is apparently not - error out
        raise DNSConversionError(
            f'A backslash must not be followed by "{to_text(letter)}" (index {index})'
        )
    if index + 1 >= len(value):
        # We need more letters for a three-digit decimal sequence
        seq_type = "octal" if use_octal else "decimal"
        raise DNSConversionError(
            f"The {seq_type} sequence at the end requires {index + 2 - len(value)} more digit(s)"
        )
    letter = value[index : index + 1]
    index += 1
    v1 = _DECIMAL_DIGITS.find(letter)
    if v1 < 0 or (use_octal and v1 >= 8):
        seq_type = "octal" if use_octal else "decimal"
        raise DNSConversionError(
            f'The second letter of the {seq_type} sequence at index {index} is not a {seq_type} digit, but "{to_text(letter)}"'
        )
    letter = value[index : index + 1]
    index += 1
    v0 = _DECIMAL_DIGITS.find(letter)
    if v0 < 0 or (use_octal and v0 >= 8):
        seq_type = "octal" if use_octal else "decimal"
        raise DNSConversionError(
            f'The third letter of the {seq_type} sequence at index {index} is not a {seq_type} digit, but "{to_text(letter)}"'
        )
    if use_octal:
        return (
            bytes((v2 * 64 + v1 * 8 + v0,)),
            index,
        )
    return (
        bytes((v2 * 100 + v1 * 10 + v0,)),
        index,
    )


def decode_txt_value(
    value: str | bytes, character_encoding: t.Literal["octal", "decimal"]
) -> str:
    """
    Given an encoded TXT value, decodes it.

    Raises DNSConversionError in case of errors.
    """
    if character_encoding not in ("octal", "decimal"):
        raise ValueError('character_encoding must be set to "octal" or "decimal"')
    value = to_bytes(value)
    state = _STATE_OUTSIDE
    index = 0
    length = len(value)
    result = []
    while index < length:
        letter = value[index : index + 1]
        index += 1
        if letter == b" ":
            if state == _STATE_QUOTED_STRING:
                result.append(letter)
            else:
                state = _STATE_OUTSIDE
        elif letter == b"\\":
            if state != _STATE_QUOTED_STRING:
                state = _STATE_UNQUOTED_STRING
            letter, index = _parse_quoted(value, index, character_encoding == "octal")
            result.append(letter)
        elif letter == b'"':
            if state == _STATE_QUOTED_STRING:
                state = _STATE_OUTSIDE
            elif state == _STATE_OUTSIDE:
                state = _STATE_QUOTED_STRING
            else:
                raise DNSConversionError(
                    f"Unexpected double quotation mark inside an unquoted block at position {index}"
                )
        else:
            if state != _STATE_QUOTED_STRING:
                state = _STATE_UNQUOTED_STRING
            result.append(letter)

    if state == _STATE_QUOTED_STRING:
        raise DNSConversionError("Missing double quotation mark at the end of value")

    return to_text(b"".join(result))


def _get_utf8_length(first_byte_value):
    """
    Given the byte value of a UTF-8 letter, returns the length of the UTF-8 character.
    """
    if first_byte_value & 0xE0 == 0xC0:
        return 2
    if first_byte_value & 0xF0 == 0xE0:
        return 3
    if first_byte_value & 0xF8 == 0xF0:
        return 4
    # Should not happen
    return 1


def encode_txt_value(
    value: str | bytes,
    *,
    always_quote: bool = False,
    use_character_encoding: bool = True,
    character_encoding: t.Literal["octal", "decimal"],
) -> str:
    """
    Given a decoded TXT value, encodes it.

    If always_quote is set to True, always use double quotes for all strings.
    If use_character_encoding (default: True) is set to False, do not use octal encoding.
    """
    if character_encoding not in ("octal", "decimal"):
        raise ValueError('character_encoding must be set to "octal" or "decimal"')

    value = to_bytes(value)
    buffer: list[bytes] = []  # the entries must be single character strings
    output: list[bytes] = []

    def append(buf: list[bytes]) -> None:
        value = b"".join(buf)
        if b" " in value or not value or always_quote:
            value = b'"%s"' % value
        output.append(value)

    index = 0
    length = len(value)
    while index < length:
        letter = value[index : index + 1]
        index += 1

        # Add letter
        if letter in (b'"', b"\\"):
            # Make sure that we do not split up an escape sequence over multiple TXT strings
            if len(buffer) + 2 > 255:
                append(buffer[:255])
                buffer = buffer[255:]
            buffer.append(b"\\")
            buffer.append(letter)
        elif use_character_encoding and not 0x20 <= ord(letter) < 0x7F:
            # Make sure that we do not split up a decimal sequence over multiple TXT strings
            if len(buffer) + 4 > 255:
                append(buffer[:255])
                buffer = buffer[255:]
            letter_value = ord(letter)
            buffer.append(b"\\")
            if character_encoding == "octal":
                v2 = (letter_value >> 6) & 7
                v1 = (letter_value >> 3) & 7
                v0 = letter_value & 7
            else:
                v2 = (letter_value // 100) % 10
                v1 = (letter_value // 10) % 10
                v0 = letter_value % 10
            buffer.append(_DECIMAL_DIGITS[v2 : v2 + 1])
            buffer.append(_DECIMAL_DIGITS[v1 : v1 + 1])
            buffer.append(_DECIMAL_DIGITS[v0 : v0 + 1])
        elif not use_character_encoding and (ord(letter) & 0x80) != 0:
            utf8_length = min(_get_utf8_length(ord(letter)), length - index + 1)
            # Make sure that we do not split up a UTF-8 letter over multiple TXT strings
            if len(buffer) + utf8_length > 255:
                append(buffer[:255])
                buffer = buffer[255:]
            buffer.append(letter)
            while utf8_length > 1:
                buffer.append(value[index : index + 1])
                index += 1
                utf8_length -= 1
        else:
            buffer.append(letter)

        # Split if too long
        if len(buffer) >= 255:
            append(buffer[:255])
            buffer = buffer[255:]

    if buffer or not output:
        append(buffer)

    return to_text(b" ".join(output))



#!/usr/bin/python

# Copyright (c) 2023, Gabriele Pongelli (gabriele.pongelli@gmail.com)
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: gitlab_label
short_description: Creates/updates/deletes GitLab Labels belonging to project or group
version_added: 8.3.0
description:
  - When a label does not exist, it is created.
  - When a label does exist, its value is updated when the values are different.
  - Labels can be purged.
author:
  - "Gabriele Pongelli (@gpongelli)"
requirements:
  - python-gitlab python module
extends_documentation_fragment:
  - community.general._auth_basic
  - community.general._gitlab
  - community.general._attributes

attributes:
  check_mode:
    support: full
  diff_mode:
    support: none

options:
  state:
    description:
      - Create or delete project or group label.
    default: present
    type: str
    choices: ["present", "absent"]
  purge:
    description:
      - When set to V(true), delete all labels which are not mentioned in the task.
    default: false
    type: bool
  project:
    description:
      - The path and name of the project. Either this or O(group) is required.
    type: str
  group:
    description:
      - The path of the group. Either this or O(project) is required.
    type: str
  labels:
    description:
      - A list of dictionaries that represents gitlab project's or group's labels.
    type: list
    elements: dict
    default: []
    suboptions:
      name:
        description:
          - The name of the label.
        type: str
        required: true
      color:
        description:
          - The color of the label.
          - Required when O(state=present).
        type: str
      priority:
        description:
          - Integer value to give priority to the label.
        type: int
        default:
      description:
        description:
          - Label's description.
        type: str
        default:
      new_name:
        description:
          - Optional field to change label's name.
        type: str
        default:
"""


EXAMPLES = r"""
# same project's task can be executed for group
- name: Create one Label
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
        color: "#123456"
    state: present

- name: Create many group labels
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    group: "group1"
    labels:
      - name: label_one
        color: "#123456"
        description: this is a label
        priority: 20
      - name: label_two
        color: "#554422"
    state: present

- name: Create many project labels
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
        color: "#123456"
        description: this is a label
        priority: 20
      - name: label_two
        color: "#554422"
    state: present

- name: Set or update some labels
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
        color: "#224488"
    state: present

- name: Add label in check mode
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
        color: "#224488"
    check_mode: true

- name: Delete Label
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
    state: absent

- name: Change Label name
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    labels:
      - name: label_one
        new_name: label_two
    state: absent

- name: Purge all labels
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    purge: true

- name: Delete many labels
  community.general.gitlab_label:
    api_url: https://gitlab.com
    api_token: secret_access_token
    project: "group1/project1"
    state: absent
    labels:
      - name: label-abc123
      - name: label-two
"""

RETURN = r"""
labels:
  description: Four lists of the labels which were added, updated, removed or exist.
  returned: success
  type: dict
  contains:
    added:
      description: A list of labels which were created.
      returned: always
      type: list
      sample: ["abcd", "label-one"]
    untouched:
      description: A list of labels which exist.
      returned: always
      type: list
      sample: ["defg", "new-label"]
    removed:
      description: A list of labels which were deleted.
      returned: always
      type: list
      sample: ["defg", "new-label"]
    updated:
      description: A list pre-existing labels whose values have been set.
      returned: always
      type: list
      sample: ["defg", "new-label"]
labels_obj:
  description: API object.
  returned: success
  type: dict
"""

from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._gitlab import (
    auth_argument_spec,
    ensure_gitlab_package,
    find_group,
    find_project,
    gitlab_authentication,
)


class GitlabLabels:
    def __init__(self, module, gitlab_instance, group_id, project_id):
        self._gitlab = gitlab_instance
        self.gitlab_object = group_id if group_id else project_id
        self.is_group_label = True if group_id else False
        self._module = module

    def list_all_labels(self):
        page_nb = 1
        labels = []
        vars_page = self.gitlab_object.labels.list(page=page_nb)
        while len(vars_page) > 0:
            labels += vars_page
            page_nb += 1
            vars_page = self.gitlab_object.labels.list(page=page_nb)
        return labels

    def create_label(self, var_obj):
        if self._module.check_mode:
            return True, True

        var = {
            "name": var_obj.get("name"),
            "color": var_obj.get("color"),
        }

        if var_obj.get("description") is not None:
            var["description"] = var_obj.get("description")

        if var_obj.get("priority") is not None:
            var["priority"] = var_obj.get("priority")

        _obj = self.gitlab_object.labels.create(var)
        return True, _obj.asdict()

    def update_label(self, var_obj):
        if self._module.check_mode:
            return True, True
        _label = self.gitlab_object.labels.get(var_obj.get("name"))

        if var_obj.get("new_name") is not None:
            _label.new_name = var_obj.get("new_name")

        if var_obj.get("description") is not None:
            _label.description = var_obj.get("description")
        if var_obj.get("priority") is not None:
            _label.priority = var_obj.get("priority")
        if var_obj.get("color") is not None:
            _label.color = var_obj.get("color")

        # save returns None
        _label.save()
        return True, _label.asdict()

    def delete_label(self, var_obj):
        if self._module.check_mode:
            return True, True
        _label = self.gitlab_object.labels.get(var_obj.get("name"))
        # delete returns None
        _label.delete()
        return True, _label.asdict()


def compare(requested_labels, existing_labels, state):
    # we need to do this, because it was determined in a previous version - more or less buggy
    # basically it is not necessary and might result in more/other bugs!
    # but it is required  and only relevant for check mode!!
    # logic represents state 'present' when not purge. all other can be derived from that
    # untouched => equal in both
    # updated => name and scope are equal
    # added => name and scope does not exist
    untouched = list()
    updated = list()
    added = list()

    if state == "present":
        _existing_labels = list()
        for item in existing_labels:
            _existing_labels.append({"name": item.get("name")})

        for var in requested_labels:
            if var in existing_labels:
                untouched.append(var)
            else:
                compare_item = {"name": var.get("name")}
                if compare_item in _existing_labels:
                    updated.append(var)
                else:
                    added.append(var)

    return untouched, updated, added


def native_python_main(this_gitlab, purge, requested_labels, state, module):
    change = False
    return_value = dict(added=[], updated=[], removed=[], untouched=[])
    return_obj = dict(added=[], updated=[], removed=[])

    labels_before = [x.asdict() for x in this_gitlab.list_all_labels()]

    # filter out and enrich before compare
    for item in requested_labels:
        # add defaults when not present
        if item.get("description") is None:
            item["description"] = ""
        if item.get("new_name") is None:
            item["new_name"] = None
        if item.get("priority") is None:
            item["priority"] = None

        # group label does not have priority, removing for comparison
        if this_gitlab.is_group_label:
            item.pop("priority")

    for item in labels_before:
        # remove field only from server
        item.pop("id")
        item.pop("description_html")
        item.pop("text_color")
        item.pop("subscribed")
        # field present only when it is a project's label
        if "is_project_label" in item:
            item.pop("is_project_label")
        item["new_name"] = None

    if state == "present":
        add_or_update = [x for x in requested_labels if x not in labels_before]
        for item in add_or_update:
            try:
                _rv, _obj = this_gitlab.create_label(item)
                if _rv:
                    return_value["added"].append(item)
                    return_obj["added"].append(_obj)
            except Exception:
                # create raises exception with following error message when label already exists
                _rv, _obj = this_gitlab.update_label(item)
                if _rv:
                    return_value["updated"].append(item)
                    return_obj["updated"].append(_obj)

        if purge:
            # re-fetch
            _labels = this_gitlab.list_all_labels()

            for item in labels_before:
                _rv, _obj = this_gitlab.delete_label(item)
                if _rv:
                    return_value["removed"].append(item)
                    return_obj["removed"].append(_obj)

    elif state == "absent":
        if not purge:
            _label_names_requested = [x["name"] for x in requested_labels]
            remove_requested = [x for x in labels_before if x["name"] in _label_names_requested]
            for item in remove_requested:
                _rv, _obj = this_gitlab.delete_label(item)
                if _rv:
                    return_value["removed"].append(item)
                    return_obj["removed"].append(_obj)
        else:
            for item in labels_before:
                _rv, _obj = this_gitlab.delete_label(item)
                if _rv:
                    return_value["removed"].append(item)
                    return_obj["removed"].append(_obj)

    if module.check_mode:
        _untouched, _updated, _added = compare(requested_labels, labels_before, state)
        return_value = dict(added=_added, updated=_updated, removed=return_value["removed"], untouched=_untouched)

    if any(return_value[x] for x in ["added", "removed", "updated"]):
        change = True

    labels_after = [x.asdict() for x in this_gitlab.list_all_labels()]

    return change, return_value, labels_before, labels_after, return_obj


def main():
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(auth_argument_spec())
    argument_spec.update(
        project=dict(type="str"),
        group=dict(type="str"),
        purge=dict(type="bool", default=False),
        labels=dict(
            type="list",
            elements="dict",
            default=list(),
            options=dict(
                name=dict(type="str", required=True),
                color=dict(type="str"),
                description=dict(type="str"),
                priority=dict(type="int"),
                new_name=dict(type="str"),
            ),
        ),
        state=dict(type="str", default="present", choices=["absent", "present"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        mutually_exclusive=[
            ["api_username", "api_token"],
            ["api_username", "api_oauth_token"],
            ["api_username", "api_job_token"],
            ["api_token", "api_oauth_token"],
            ["api_token", "api_job_token"],
            ["project", "group"],
        ],
        required_together=[
            ["api_username", "api_password"],
        ],
        required_one_of=[["api_username", "api_token", "api_oauth_token", "api_job_token"], ["project", "group"]],
        supports_check_mode=True,
    )
    ensure_gitlab_package(module)

    gitlab_project = module.params["project"]
    gitlab_group = module.params["group"]
    purge = module.params["purge"]
    label_list = module.params["labels"]
    state = module.params["state"]

    gitlab_instance = gitlab_authentication(module, min_version="3.2.0")

    # find_project can return None, but the other must exist
    gitlab_project_id = find_project(gitlab_instance, gitlab_project)

    # find_group can return None, but the other must exist
    gitlab_group_id = find_group(gitlab_instance, gitlab_group)

    # if both not found, module must exist
    if not gitlab_project_id and not gitlab_group_id:
        if gitlab_project and not gitlab_project_id:
            module.fail_json(msg=f"project '{gitlab_project}' not found.")
        if gitlab_group and not gitlab_group_id:
            module.fail_json(msg=f"group '{gitlab_group}' not found.")

    this_gitlab = GitlabLabels(
        module=module, gitlab_instance=gitlab_instance, group_id=gitlab_group_id, project_id=gitlab_project_id
    )

    if state == "present":
        _existing_labels = [x.asdict()["name"] for x in this_gitlab.list_all_labels()]

        # color is mandatory when creating label, but it is optional when changing name or updating other fields
        if any(x["color"] is None and x["new_name"] is None and x["name"] not in _existing_labels for x in label_list):
            module.fail_json(msg="color parameter is required for new labels")

    change, raw_return_value, before, after, _obj = native_python_main(this_gitlab, purge, label_list, state, module)

    if not module.check_mode:
        raw_return_value["untouched"] = [x for x in before if x in after]

    added = [x.get("name") for x in raw_return_value["added"]]
    updated = [x.get("name") for x in raw_return_value["updated"]]
    removed = [x.get("name") for x in raw_return_value["removed"]]
    untouched = [x.get("name") for x in raw_return_value["untouched"]]
    return_value = dict(added=added, updated=updated, removed=removed, untouched=untouched)

    module.exit_json(changed=change, labels=return_value, labels_obj=_obj)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2025, Dev Sinha (@DevSinha13) <devsinh@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)
from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "certified"}

DOCUMENTATION = r"""
---
module: aci_vrf_fallback_route_group
version_added: "2.12.0"
short_description: Manage VRF Fallback Route Groups (fv:FBRGroup, fv:FBRoute, and fv:FBRMember)
description:
- Manage VRF Fallback Route Groups on Cisco ACI fabrics.
- Fallback Route Groups are used to specify routes and next-hop addresses for VRFs.
options:
  tenant:
    description:
    - The name of the Tenant associated with the VRF Fallback Route Group.
    type: str
    aliases: [ tenant_name ]
  vrf:
    description:
    - The name of the VRF associated with the VRF Fallback Route Group.
    type: str
    aliases: [ context, vrf_name ]
  name:
    description:
    - The name of the VRF Fallback Route Group.
    type: str
    aliases: [ vrf_fallback_route_group ]
  prefix_address:
    description:
    - The fallback route (prefix address) for the VRF Fallback Route Group.
    - If not specified, the existing fallback route will remain unchanged.
    - To delete the fallback route, pass an empty string as the attribute value.
    type: str
    aliases: [ fallback_route ]
  fallback_members:
    description:
    - A list of fallback member IP addresses (next-hop addresses) for the VRF Fallback Route Group.
    - Members not in the list will be removed from the configuration.
    - If not specified, the existing fallback members will remain unchanged.
    - To delete all the fallback members, pass an empty list.
    type: list
    elements: str
    aliases: [ next_hop_address ]
  description:
    description:
    - The description for the VRF Fallback Route Group.
    type: str
    aliases: [ descr ]
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
extends_documentation_fragment:
- cisco.aci.aci
- cisco.aci.annotation
- cisco.aci.owner

notes:
- The C(tenant) and C(vrf) must exist before using this module.
  Use the M(cisco.aci.aci_tenant) and M(cisco.aci.aci_vrf) modules to create them if needed.
seealso:
- module: cisco.aci.aci_vrf
- module: cisco.aci.aci_tenant
- name: APIC Management Information Model reference
  description: More information about the internal APIC class B(fv:FBRGroup), B(fv:FBRoute), and B(fv:FBRMember).
  link: https://developer.cisco.com/docs/apic-mim-ref/
author:
- Dev Sinha (@devsinha13)
"""

EXAMPLES = r"""
- name: Create a new VRF Fallback Route Group
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: ansible_test
    vrf: vrf_test
    vrf_fallback_route_group: test_fallback_route_group
    fallback_route: 1.1.1.1/24
    fallback_members:
      - 192.168.1.1
      - 192.168.1.2
    description: Test Fallback Route Group
    state: present
  delegate_to: localhost

- name: Update fallback members in an existing VRF Fallback Route Group
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: ansible_test
    vrf: vrf_test
    vrf_fallback_route_group: test_fallback_route_group
    fallback_members:
      - 192.168.1.1
      - 192.168.1.3
    state: present
  delegate_to: localhost

- name: Delete children for VRF Fallback Route Group
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: ansible_test
    vrf: vrf_test
    vrf_fallback_route_group: test_fallback_route_group
    fallback_route: ""
    fallback_members: []
    state: present
  delegate_to: localhost

- name: Query a VRF Fallback Route Group
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: ansible_test
    vrf: vrf_test
    vrf_fallback_route_group: test_fallback_route_group
    state: query
  delegate_to: localhost
  register: query_result

- name: Query all VRF Fallback Route Groups
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    state: query
  delegate_to: localhost
  register: query_result

- name: Delete a VRF Fallback Route Group
  cisco.aci.aci_vrf_fallback_route_group:
    host: apic
    username: admin
    password: SomeSecretPassword
    tenant: ansible_test
    vrf: vrf_test
    vrf_fallback_route_group: test_fallback_route_group
    state: absent
  delegate_to: localhost
"""

RETURN = r"""
current:
  description: The existing configuration from the APIC after the module has finished
  returned: success
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production environment",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
error:
  description: The error information as returned from the APIC
  returned: failure
  type: dict
  sample:
    {
        "code": "122",
        "text": "unknown managed object class foo"
    }
raw:
  description: The raw output returned by the APIC REST API (xml or json)
  returned: parse error
  type: str
  sample: '<?xml version="1.0" encoding="UTF-8"?><imdata totalCount="1"><error code="122" text="unknown managed object class foo"/></imdata>'
sent:
  description: The actual/minimal configuration pushed to the APIC
  returned: info
  type: list
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment"
            }
        }
    }
previous:
  description: The original configuration from the APIC before the module has started
  returned: info
  type: list
  sample:
    [
        {
            "fvTenant": {
                "attributes": {
                    "descr": "Production",
                    "dn": "uni/tn-production",
                    "name": "production",
                    "nameAlias": "",
                    "ownerKey": "",
                    "ownerTag": ""
                }
            }
        }
    ]
proposed:
  description: The assembled configuration from the user-provided parameters
  returned: info
  type: dict
  sample:
    {
        "fvTenant": {
            "attributes": {
                "descr": "Production environment",
                "name": "production"
            }
        }
    }
filter_string:
  description: The filter string used for the request
  returned: failure or debug
  type: str
  sample: ?rsp-prop-include=config-only
method:
  description: The HTTP method used for the request to the APIC
  returned: failure or debug
  type: str
  sample: POST
response:
  description: The HTTP response from the APIC
  returned: failure or debug
  type: str
  sample: OK (30 bytes)
status:
  description: The HTTP status from the APIC
  returned: failure or debug
  type: int
  sample: 200
url:
  description: The HTTP url used for the request to the APIC
  returned: failure or debug
  type: str
  sample: https://10.11.12.13/api/mo/uni/tn-production.json
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.aci.plugins.module_utils.aci import ACIModule, aci_argument_spec, aci_annotation_spec, aci_owner_spec


def main():
    argument_spec = aci_argument_spec()
    argument_spec.update(aci_annotation_spec())
    argument_spec.update(aci_owner_spec())
    argument_spec.update(
        tenant=dict(type="str", aliases=["tenant_name"]),
        vrf=dict(type="str", aliases=["context", "vrf_name"]),
        vrf_fallback_route_group=dict(type="str", aliases=["name"]),
        fallback_route=dict(type="str", aliases=["prefix_address"]),
        fallback_members=dict(type="list", elements="str", aliases=["next_hop_address"]),
        description=dict(type="str", aliases=["descr"]),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["tenant", "vrf"]],
            ["state", "present", ["tenant", "vrf"]],
        ],
    )

    tenant = module.params.get("tenant")
    vrf = module.params.get("vrf")
    vrf_fallback_route_group = module.params.get("vrf_fallback_route_group")
    fallback_route = module.params.get("fallback_route")
    fallback_members = module.params.get("fallback_members")
    description = module.params.get("description")
    state = module.params.get("state")

    aci = ACIModule(module)

    aci.construct_url(
        root_class=dict(
            aci_class="fvTenant",
            aci_rn="tn-{0}".format(tenant),
            module_object=tenant,
            target_filter={"name": tenant},
        ),
        subclass_1=dict(
            aci_class="fvCtx",
            aci_rn="ctx-{0}".format(vrf),
            module_object=vrf,
            target_filter={"name": vrf},
        ),
        subclass_2=dict(
            aci_class="fvFBRGroup",
            aci_rn="fbrg-{0}".format(vrf_fallback_route_group),
            module_object=vrf_fallback_route_group,
            target_filter={"name": vrf_fallback_route_group},
        ),
        child_classes=["fvFBRMember", "fvFBRoute"],
    )

    aci.get_existing()

    if state == "present":

        child_configs = []

        existing_members = []
        existing_route = None

        if isinstance(aci.existing, list) and len(aci.existing) > 0:
            for child in aci.existing[0].get("fvFBRGroup", {}).get("children", {}):
                existing_member = child.get("fvFBRMember", {}).get("attributes", {}).get("rnhAddr")
                if existing_member:
                    existing_members.append(existing_member)
                route = child.get("fvFBRoute", {}).get("attributes", {}).get("fbrPrefix")
                if route:
                    existing_route = route

        if fallback_members is not None:
            fallback_members_set = set(fallback_members)

            existing_members_set = set(existing_members)

            for member in fallback_members_set - existing_members_set:
                child_configs.append(dict(fvFBRMember=dict(attributes=dict(rnhAddr=member))))
            for existing_member in existing_members_set - fallback_members_set:
                child_configs.append(dict(fvFBRMember=dict(attributes=dict(rnhAddr=existing_member, status="deleted"))))

        if fallback_route is not None and fallback_route != existing_route:
            if existing_route:
                # Appending to child_config list not possible because of APIC Error 182: Multiple fallback routes not allowed in one group.
                # A seperate delete request to dn of the fvFBRoute is needed to remove the object prior to adding to child_configs.
                # Failed child_config is displayed in below:
                # child_configs.append(
                #     dict(fvFBRoute=dit(attributes=dict(fbrPrefix=existing_route, status="deleted"))),
                # )
                aci.api_call(
                    "DELETE",
                    "{0}/api/mo/uni/tn-{1}/ctx-{2}/fbrg-{3}/pfx-[{4}].json".format(aci.base_url, tenant, vrf, vrf_fallback_route_group, existing_route),
                )
            if fallback_route:
                child_configs.append(
                    dict(fvFBRoute=dict(attributes=dict(fbrPrefix=fallback_route))),
                )

        aci.payload(
            aci_class="fvFBRGroup",
            class_config=dict(
                descr=description,
                name=vrf_fallback_route_group,
            ),
            child_configs=child_configs,
        )

        aci.get_diff(aci_class="fvFBRGroup")

        aci.post_config()
    elif state == "absent":
        aci.delete_config()

    aci.exit_json()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2017, René Moser <mail@renemoser.net>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: influxdb_query
short_description: Query data points from InfluxDB
description:
  - Query data points from InfluxDB.
author: "René Moser (@resmo)"
requirements:
  - "influxdb >= 0.9"
attributes:
  check_mode:
    support: full
  diff_mode:
    support: none
options:
  query:
    description:
      - Query to be executed.
    required: true
    type: str
  database_name:
    description:
      - Name of the database.
    required: true
    type: str
extends_documentation_fragment:
  - community.general._influxdb
  - community.general._attributes
"""

EXAMPLES = r"""
- name: Query connections
  community.general.influxdb_query:
    hostname: "{{ influxdb_ip_address }}"
    database_name: "{{ influxdb_database_name }}"
    query: "select mean(value) from connections"
  register: connection

- name: Query connections with tags filters
  community.general.influxdb_query:
    hostname: "{{ influxdb_ip_address }}"
    database_name: "{{ influxdb_database_name }}"
    query: "select mean(value) from connections where region='zue01' and host='server01'"
  register: connection

- name: Print results from the query
  ansible.builtin.debug:
    var: connection.query_results
"""

RETURN = r"""
query_results:
  description: Result from the query.
  returned: success
  type: list
  sample:
    - mean: 1245.5333333333333
      time: "1970-01-01T00:00:00Z"
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.general.plugins.module_utils._influxdb import InfluxDb


class AnsibleInfluxDBRead(InfluxDb):
    def read_by_query(self, query):
        client = self.connect_to_influxdb()
        try:
            rs = client.query(query)
            if rs:
                return list(rs.get_points())
        except Exception as e:
            self.module.fail_json(msg=f"{e}")


def main():
    argument_spec = InfluxDb.influxdb_argument_spec()
    argument_spec.update(
        query=dict(type="str", required=True),
        database_name=dict(required=True, type="str"),
    )
    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    influx = AnsibleInfluxDBRead(module)
    query = module.params.get("query")
    results = influx.read_by_query(query)
    module.exit_json(changed=True, query_results=results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# Copyright (c) 2025 Markus Bergholz
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later
from __future__ import annotations

DOCUMENTATION = r"""
module: adguardhome_rewrite

short_description: Add, update or delete DNS rewrite rules from AdGuardHome

version_added: 3.3.0

description:
  - Add, update or delete DNS rewrite rules from AdGuardHome.
extends_documentation_fragment:
  - community.dns._adguardhome.connectivity
  - community.dns._attributes
options:
  state:
    description:
      - Wether a rewrite rule should be added/updated (O(state=present)) or removed (O(state=absent)).
    type: str
    default: present
    choices:
      - present
      - absent
  domain:
    description:
      - Domain or wildcard domain that you want to be rewritten by AdGuardHome.
    type: str
    required: true
  answer:
    description:
      - Value for the domain rewrite.
      - Required when O(state=present).
      - Value can be a CNAME, A or AAAA record.
    type: str
    required: false
attributes:
  check_mode:
    support: full
  diff_mode:
    support: full
  idempotent:
    support: full


author:
  - Markus Bergholz (@markuman) <markuman+spambelongstogoogle@gmail.com>
"""

EXAMPLES = r"""
- name: Add DNS rewrite rule in AdGuardHome
  community.dns.adguardhome_rewrite:
    state: present
    answer: 127.0.0.1
    domain: example.org

# When removing a rewrite, the current answer value must also match.
# Therefore you can just leave it out and the existing value
# will be used.
- name: Remove rewrite for example.org
  community.dns.adguardhome_rewrite:
    state: absent
    domain: example.org
"""

RETURN = r"""
rules:
  description: The modified list of rewrite rules afte rewrite rule is applied.
  type: list
  elements: dict
  returned: success
  contains:
    answer:
      description: Value of the rewrite.
      type: str
      sample: 192.168.178.71
    domain:
      description: Domain of the rewrite.
      type: str
      sample: dns.osuv.de
"""

import typing as t

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.community.dns.plugins.module_utils._adguardhome.api import (
    AdGuardHomeAPIHandler,
    create_adguardhome_argument_spec,
)
from ansible_collections.community.dns.plugins.module_utils._argspec import ArgumentSpec


def find_and_compare(
    rules: list[dict[str, t.Any]], domain: str, answer: str
) -> tuple[bool, bool, dict[str, t.Any]]:
    domain_exists = False
    value_is_different = False
    target = {}
    for rule in rules:
        if rule["domain"] == domain:
            domain_exists = True
            target = {"domain": domain, "answer": rule["answer"]}
            if rule["answer"] != answer:
                value_is_different = True
            break
    return domain_exists, value_is_different, target


def main() -> None:
    argument_spec = ArgumentSpec(
        {
            "state": {
                "type": "str",
                "default": "present",
                "choices": ["present", "absent"],
            },
            "answer": {"type": "str", "required": False},
            "domain": {"type": "str", "required": True},
        },
        required_if=[("state", "present", ["answer"])],
    )
    argument_spec.merge(create_adguardhome_argument_spec())
    module = AnsibleModule(supports_check_mode=True, **argument_spec.to_kwargs())

    domain = module.params.get("domain")
    answer = module.params.get("answer")
    state = module.params.get("state")

    adguardhome = AdGuardHomeAPIHandler(module.params, module.fail_json)

    before = (
        adguardhome.list()
    )  # Note that this is updated to the 'after' value in check mode (but not outside of check mode!)
    changed = False

    domain_exists, value_is_different, target = find_and_compare(before, domain, answer)
    if state == "present":
        if not domain_exists and not value_is_different:
            changed = True
            if module.check_mode:
                before = before + [{"answer": answer, "domain": domain}]
            else:
                adguardhome.add_or_delete(domain, answer, "add", target)

        if domain_exists and value_is_different:
            changed = True
            if module.check_mode:
                for item in before:
                    if item["domain"] == "example.org":
                        item["value"] = answer
                        break
            else:
                adguardhome.update(domain, answer, target)

    else:
        if domain_exists:
            changed = True
            if module.check_mode:
                before = [item for item in before if item["domain"] != domain]
            else:
                adguardhome.add_or_delete(domain, answer, "delete", target)

    after = adguardhome.list()

    if module.check_mode:
        return_rules = before
        diff_item = {"before": {"rules": after}, "after": {"rules": before}}
    else:
        return_rules = after
        diff_item = {"before": {"rules": before}, "after": {"rules": after}}

    module.exit_json(changed=changed, diff=diff_item, rules=return_rules)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_hsrp_interfaces
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_hsrp_interfaces
short_description: HSRP interfaces resource module
description: Resource module to configure HSRP on interfaces.
version_added: 10.1.0
author:
  - Chris Van Heuveln (@chrisvanheuveln)
  - Sagar Paul (@KB-perByte)
notes:
  - Tested against NX-OS 10.4(2) Nexus 9000v.
  - Feature bfd and hsrp, should be enabled for this module.
  - Unsupported for Cisco MDS
options:
  running_config:
    description:
      - This option is used only with state I(parsed).
      - The value of this option should be the output received from the NX-OS device
        by executing the command B(show running-config | section '^interface').
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into Ansible structured data as per the resource module's argspec
        and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A dictionary of HSRP configuration options to add to interface
    type: list
    elements: dict
    suboptions:
      name:
        type: str
        description: The name of the interface.
      bfd:
        type: str
        description:
          - Enable/Disable HSRP Bidirectional Forwarding Detection (BFD) on the interface.
          - B(Deprecated), Use standby.bfd instead, the facts would always render bfd information
            as a part of standby configuration
          - This option has been deprecated and will be removed in a release after 2028-06-01.
        choices:
          - enable
          - disable
      standby_options:
        description: Group number and group options for standby (HSRP)
        type: list
        elements: dict
        suboptions:
          group_no:
            description: Group number
            type: int
          authentication:
            description: Authentication configuration
            type: dict
            suboptions:
              key_chain:
                description: Set key chain
                type: str
              key_string:
                description: Set key string
                type: str
              password_text:
                description: Password text valid for plain text and and key-string
                type: str
          follow:
            description: Groups to be followed
            type: str
          mac_address:
            description: Virtual MAC address
            type: str
          ip:
            description: Enable HSRP IPv4 and set the virtual IP address
            type: list
            elements: dict
            suboptions:
              virtual_ip:
                description: Virtual IP address
                type: str
              secondary:
                description: Make this IP address a secondary virtual IP address
                type: bool
          group_name:
            description: Redundancy name string
            type: str
          preempt:
            description: Overthrow lower priority Active routers
            type: dict
            suboptions:
              minimum:
                description: Delay at least this long
                type: int
              reload:
                description: Delay after reload
                type: int
              sync:
                description: Wait for IP redundancy clients
                type: int
          priority:
            description: Priority level
            type: dict
            suboptions:
              level:
                description: Priority level value
                type: int
                default: 100
              upper:
                description: Set upper threshold value (forwarding-threshold)
                type: int
              lower:
                description: Set lower threshold value (forwarding-threshold)
                type: int
          timer:
            description: Overthrow lower priority Active routers
            type: dict
            suboptions:
              hello_interval:
                description: Hello interval in seconds
                type: int
              hold_time:
                description: Hold time in seconds
                type: int
              msec:
                description: Specify hello interval in milliseconds
                type: bool
          track:
            description: Priority tracking
            type: list
            elements: dict
            suboptions:
              object_no:
                description: Track object number
                type: int
              decrement:
                description: Priority decrement
                type: int
      standby:
        description:
          - Standby options generic, not idempotent when version 1 (HSRP)
        type: dict
        suboptions:
          bfd:
            description: Enable HSRP BFD
            type: bool
          delay:
            description: HSRP initialization delay
            type: dict
            suboptions:
              minimum:
                description: Delay at least this long
                type: int
              reload:
                description: Delay after reload
                type: int
          mac_refresh:
            description: Refresh MAC cache on switch by periodically sending packet from virtual mac address
            type: int
          use_bia:
            description: HSRP uses interface's burned in address (does not work with mac address)
            type: dict
            suboptions:
              set:
                description: Set use-bia only
                type: bool
              scope:
                description: Scope interface option (hsrp use-bia scope interface)
                type: bool
          version:
            description: HSRP version
            type: int
            choices:
              - 1
              - 2
  state:
    choices:
      - merged
      - replaced
      - overridden
      - deleted
      - rendered
      - gathered
      - parsed
    default: merged
    description:
      - The state the configuration should be left in
      - The states I(rendered), I(gathered) and I(parsed) does not perform any change
        on the device.
      - The state I(rendered) will transform the configuration in C(config) option to
        platform specific CLI commands which will be returned in the I(rendered) key
        within the result. For state I(rendered) active connection to remote host is
        not required.
      - The state I(gathered) will fetch the running configuration from device and transform
        it into structured data in the format as per the resource module argspec and
        the value is returned in the I(gathered) key within the result.
      - The state I(parsed) reads the configuration from C(running_config) option and
        transforms it into JSON format as per the resource module parameters and the
        value is returned in the I(parsed) key within the result. The value of C(running_config)
        option should be the same format as the output of command
        I(show running-config | section ^interface) executed on device. For state I(parsed) active
        connection to remote host is not required.
    type: str
"""

EXAMPLES = """
# Using merged

# Before state:
# -------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
# interface Ethernet1/2
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

- name: Merge provided configuration with device configuration
  cisco.nxos.nxos_hsrp_interfaces:
    config:
      - name: Ethernet1/1
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_string: SECUREKEY10
            group_name: VLAN10-GROUP
            group_no: 10
            ip:
              - secondary: true
                virtual_ip: 10.10.10.2
            mac_address: 00CC.10DD.10EE
    state: merged

# Task Output
# -----------
#
# before:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
# - name: Ethernet1/2
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7
#  commands:
# - interface Ethernet1/1
# - hsrp version 2
# - hsrp mac-refresh 400
# - hsrp 10
# - mac-address 00CC.10DD.10EE
# - name VLAN10-GROUP
# - authentication md5 key-string SECUREKEY10
# - ip 10.10.10.2 secondary
# - interface Ethernet1/2
# - hsrp bfd
# - hsrp version 2
# - hsrp mac-refresh 400
# - hsrp 20
# - mac-address 00CC.10DD.10EF
# - name VLAN20-GROUP
# - authentication md5 key-chain SECUREKEY20
# - ip 10.10.10.3 secondary
#  after:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN10-GROUP
#       group_no: 10
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.2
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.3
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7

# After state:
# ------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 10
#     authentication md5 key-string SECUREKEY10
#     name VLAN10-GROUP
#     mac-address 00CC.10DD.10EE
#     ip 10.10.10.2 secondary
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
#     ip 10.10.10.3 secondary
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

# Using replaced

# Before state:
# -------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 10
#     authentication md5 key-string SECUREKEY10
#     name VLAN10-GROUP
#     mac-address 00CC.10DD.10EE
#     ip 10.10.10.2 secondary
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
#     ip 10.10.10.3 secondary
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

- name: Replaces device configuration of listed interfaces with provided configuration
  cisco.nxos.nxos_hsrp_interfaces:
    config:
      - name: Ethernet1/1
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_string: SECUREKEY10
            group_name: VLAN11-GROUP
            group_no: 11
            mac_address: 00CC.10DD.10EE
      - name: Ethernet1/2
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_chain: SECUREKEY20
            group_name: VLAN20-GROUP
            group_no: 20
            mac_address: 00CC.10DD.10EF
    state: replaced

# Task Output
# -----------
#
#  before:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN10-GROUP
#       group_no: 10
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.2
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.3
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7
#  commands:
# - interface Ethernet1/1
# - hsrp 11
# - mac-address 00CC.10DD.10EE
# - name VLAN11-GROUP
# - authentication md5 key-string SECUREKEY10
# - no hsrp 10
# - interface Ethernet1/2
# - hsrp 20
# - no ip 10.10.10.3 secondary
#  after:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN11-GROUP
#       group_no: 11
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7


# After state:
# ------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 11
#     authentication md5 key-string SECUREKEY10
#     name VLAN11-GROUP
#     mac-address 00CC.10DD.10EE
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

# Using overridden

# Before state:
# -------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 10
#     authentication md5 key-string SECUREKEY10
#     name VLAN10-GROUP
#     mac-address 00CC.10DD.10EE
#     ip 10.10.10.2 secondary
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
#     ip 10.10.10.3 secondary

- name: Override device configuration of all interfaces with provided configuration
  cisco.nxos.nxos_hsrp_interfaces:
    config:
      - name: Ethernet1/1
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_string: SECUREKEY10
            group_name: VLAN11-GROUP
            group_no: 11
            mac_address: 00CC.10DD.10EE
      - name: Ethernet1/2
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_chain: SECUREKEY20
            group_name: VLAN20-GROUP
            group_no: 20
            mac_address: 00CC.10DD.10EF
    state: overridden

# Task Output
# -----------
#
#  before:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN10-GROUP
#       group_no: 10
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.2
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.3
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7
#  commands:
# - interface Ethernet1/1
# - hsrp 11
# - mac-address 00CC.10DD.10EE
# - name VLAN11-GROUP
# - authentication md5 key-string SECUREKEY10
# - no hsrp 10
# - interface Ethernet1/2
# - hsrp 20
# - no ip 10.10.10.3 secondary
#  after:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN11-GROUP
#       group_no: 11
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7

# After state:
# ------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 11
#     authentication md5 key-string SECUREKEY10
#     name VLAN11-GROUP
#     mac-address 00CC.10DD.10EE
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7


# Using deleted

# Before state:
# -------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 10
#     authentication md5 key-string SECUREKEY10
#     name VLAN10-GROUP
#     mac-address 00CC.10DD.10EE
#     ip 10.10.10.2 secondary
# interface Ethernet1/2
#   no switchport
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 20
#     authentication md5 key-chain SECUREKEY20
#     name VLAN20-GROUP
#     mac-address 00CC.10DD.10EF
#     ip 10.10.10.3 secondary
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

- name: Delete or return interface parameters to default settings
  cisco.nxos.nxos_hsrp_interfaces:
    config:
      - name: Ethernet1/1
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_string: SECUREKEY10
            group_name: VLAN11-GROUP
            group_no: 11
            mac_address: 00CC.10DD.10EE
      - name: Ethernet1/2
        standby:
          bfd: true
          mac_refresh: 400
          version: 2
        standby_options:
          - authentication:
              key_chain: SECUREKEY20
            group_name: VLAN20-GROUP
            group_no: 20
            mac_address: 00CC.10DD.10EF
    state: deleted

# Task Output
# -----------
#
# before:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_string: SECUREKEY10
#       group_name: VLAN10-GROUP
#       group_no: 10
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.2
#       mac_address: 00CC.10DD.10EE
# - name: Ethernet1/2
#   standby:
#     bfd: true
#     mac_refresh: 400
#     version: 2
#   standby_options:
#     - authentication:
#         key_chain: SECUREKEY20
#       group_name: VLAN20-GROUP
#       group_no: 20
#       ip:
#         - secondary: true
#           virtual_ip: 10.10.10.3
#       mac_address: 00CC.10DD.10EF
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6
# - name: Ethernet1/7
# commands:
# - interface Ethernet1/1
# - no hsrp bfd
# - no hsrp version 2
# - no hsrp mac-refresh 400
# - no hsrp 10
# - interface Ethernet1/2
# - no hsrp bfd
# - no hsrp version 2
# - no hsrp mac-refresh 400
# - no hsrp 20
# after:
# - name: Vlan1
# - name: Vlan10
# - name: Vlan14
# - name: Vlan1000
# - name: Ethernet1/1
# - name: Ethernet1/2
# - name: Ethernet1/3
# - name: Ethernet1/4
# - name: Ethernet1/5
# - name: Ethernet1/6

# After state:
# ------------
#
# switch# show running-config | section interface
# interface Vlan1
# interface Vlan10
# interface Vlan14
#   bandwidth 99999
# interface Vlan1000
# interface Ethernet1/1
#   no switchport
# interface Ethernet1/2
#   no switchport
# interface Ethernet1/3
# interface Ethernet1/4
# interface Ethernet1/5
# interface Ethernet1/6
# interface Ethernet1/7

# Using rendered

- name: Use rendered state to convert task input to device specific commands
  cisco.nxos.nxos_hsrp_interfaces:
    config:
      - name: Ethernet1/1
        description: outbound-intf
        mode: layer3
        speed: 100
      - name: Ethernet1/2
        mode: layer2
        enabled: true
        duplex: full
    state: rendered

# Task Output
# -----------
#
# rendered:
#  - interface Vlan1
#  - hsrp version 2
#  - hsrp 10
#  - timers msec 250 255
#  - authentication md5 key-chain test
#  - interface Vlan10
#  - hsrp bfd
#  - hsrp version 2
#  - hsrp mac-refresh 400
#  - hsrp 10
#  - mac-address 00CC.10DD.10EE
#  - name VLAN10-GROUP
#  - preempt delay minimum 15 reload 120 sync 10
#  - authentication md5 key-string SECUREKEY10
#  - ip 10.10.10.2 secondary
#  - interface Vlan14
#  - hsrp bfd
#  - hsrp version 2
#  - hsrp delay 22 123
#  - hsrp mac-refresh 300
#  - hsrp 14
#  - follow VLAN14-GROUP
#  - mac-address 00AA.14BB.14CC
#  - ip 192.168.14.1 secondary
#  - ip 192.168.14.2 secondary
#  - hsrp 15
#  - mac-address 00BB.14CC.15DD
#  - preempt delay minimum 10 reload 100 sync 5
#  - priority 22 forwarding-threshold lower 12 upper 22
#  - timers msec 456 33
#  - authentication md5 key-string SECUREKEY14
#  - interface Vlan1000
#  - hsrp 10
#  - mac-address 0423.4567.89AB
#  - name testhsr
#  - preempt delay minimum 33 reload 23 sync 22
#  - priority 22 forwarding-threshold lower 12 upper 22
#  - timers msec 456 33
#  - authentication md5 key-string testmesecurte
#  - ip 10.15.8.1 secondary


# Using parsed

# parsed.cfg
# ------------
#
# interface Vlan1
#   hsrp version 2
#   hsrp 10
#     authentication md5 key-chain test
#     timers msec 250  255
# interface Vlan10
#   hsrp bfd
#   hsrp version 2
#   hsrp mac-refresh 400
#   hsrp 10
#     authentication md5 key-string SECUREKEY10
#     name VLAN10-GROUP
#     mac-address 00CC.10DD.10EE
#     preempt delay minimum 15 reload 120 sync 10
#     ip 10.10.10.2 secondary
# interface Vlan14
#   bandwidth 99999
#   hsrp bfd
#   hsrp version 2
#   hsrp delay minimum 22 reload 123
#   hsrp mac-refresh 300
#   hsrp 14
#     follow VLAN14-GROUP
#     mac-address 00AA.14BB.14CC
#     ip 192.168.14.1 secondary
#     ip 192.168.14.2 secondary
#   hsrp 15
#     authentication md5 key-string SECUREKEY14
#     mac-address 00BB.14CC.15DD
#     preempt delay minimum 10 reload 100 sync 5
#     priority 22 forwarding-threshold lower 12 upper 22
#     timers msec 456  33
# interface Vlan1000
#   hsrp 10
#     authentication md5 key-string testmesecurte
#     name testhsr
#     mac-address 0423.4567.89AB
#     preempt delay minimum 33 reload 23 sync 22
#     priority 22 forwarding-threshold lower 12 upper 22
#     timers msec 456  33
#     ip 10.15.8.1 secondary

- name: Use parsed state to convert externally supplied config to structured format
  cisco.nxos.nxos_hsrp_interfaces:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output
# -----------
#
# {"parsed": [
#        {
#            "name": "Vlan1",
#            "standby": {
#                "version": 2
#            },
#            "standby_options": [
#                {
#                    "authentication": {
#                        "key_chain": "test"
#                    },
#                    "group_no": 10,
#                    "timer": {
#                        "hello_interval": 250,
#                        "hold_time": 255,
#                        "msec": true
#                    }
#                }
#            ]
#        },
#        {
#            "name": "Vlan10",
#            "standby": {
#                "bfd": true,
#                "mac_refresh": 400,
#                "version": 2
#            },
#            "standby_options": [
#                {
#                    "authentication": {
#                        "key_string": "SECUREKEY10"
#                    },
#                    "group_name": "VLAN10-GROUP",
#                    "group_no": 10,
#                    "ip": [
#                        {
#                            "secondary": true,
#                            "virtual_ip": "10.10.10.2"
#                        }
#                    ],
#                    "mac_address": "00CC.10DD.10EE",
#                    "preempt": {
#                        "minimum": 15,
#                        "reload": 120,
#                        "sync": 10
#                    }
#                }
#            ]
#        },
#        {
#            "name": "Vlan14",
#            "standby": {
#                "bfd": true,
#                "delay": {
#                    "minimum": 22,
#                    "reload": 123
#                },
#                "mac_refresh": 300,
#                "version": 2
#            },
#            "standby_options": [
#                {
#                    "follow": "VLAN14-GROUP",
#                    "group_no": 14,
#                    "ip": [
#                        {
#                            "secondary": true,
#                            "virtual_ip": "192.168.14.1"
#                        },
#                        {
#                            "secondary": true,
#                            "virtual_ip": "192.168.14.2"
#                        }
#                    ],
#                    "mac_address": "00AA.14BB.14CC"
#                },
#                {
#                    "authentication": {
#                        "key_string": "SECUREKEY14"
#                    },
#                    "group_no": 15,
#                    "mac_address": "00BB.14CC.15DD",
#                    "preempt": {
#                        "minimum": 10,
#                        "reload": 100,
#                        "sync": 5
#                    },
#                    "priority": {
#                        "level": 22,
#                        "lower": 12,
#                        "upper": 22
#                    },
#                    "timer": {
#                        "hello_interval": 456,
#                        "hold_time": 33,
#                        "msec": true
#                    }
#                }
#            ]
#        },
#        {
#            "name": "Vlan1000",
#            "standby_options": [
#                {
#                    "authentication": {
#                        "key_string": "testmesecurte"
#                    },
#                    "group_name": "testhsr",
#                    "group_no": 10,
#                    "ip": [
#                        {
#                            "secondary": true,
#                            "virtual_ip": "10.15.8.1"
#                        }
#                    ],
#                    "mac_address": "0423.4567.89AB",
#                    "preempt": {
#                        "minimum": 33,
#                        "reload": 23,
#                        "sync": 22
#                    },
#                    "priority": {
#                        "level": 22,
#                        "lower": 12,
#                        "upper": 22
#                    },
#                    "timer": {
#                        "hello_interval": 456,
#                        "hold_time": 33,
#                        "msec": true
#                    }
#                }
#            ]}
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - hsrp 14
    - follow VLAN14-GROUP
    - mac-address 00AA.14BB.14CC
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - hsrp 14
    - follow VLAN14-GROUP
    - mac-address 00AA.14BB.14CC
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.hsrp_interfaces.hsrp_interfaces import (
    Hsrp_interfacesArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.hsrp_interfaces.hsrp_interfaces import (
    Hsrp_interfaces,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=Hsrp_interfacesArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Hsrp_interfaces(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Copyright 2025 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function


__metaclass__ = type

"""
The Evpn_ethernet parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Evpn_ethernetTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Evpn_ethernetTemplate, self).__init__(lines=lines, tmplt=self, module=module)

    # fmt: off
    PARSERS = [
        {
            "name": "segment",
            "getval": re.compile(
                r"""^l2vpn\sevpn\sethernet-segment
                    (\s(?P<segment>\S+))
                    $""",
                re.VERBOSE,
            ),
            "setval": "l2vpn evpn ethernet-segment {{ segment }}",
            "result": {
                "{{ segment }}": {"segment": "{{ segment }}"},
            },
            "shared": True,
        },
        {
            "name": "df_election.wait_time",
            "getval": re.compile(
                r"""
                \s+df-election\swait-time\s(?P<wait_time>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "df-election wait-time {{ df_election.wait_time }}",
            "result": {
                "{{ segment }}": {
                    "df_election": {
                        "wait_time": "{{ wait_time }}",
                    },
                },
            },
        },
        {
            "name": "df_election.preempt_time",
            "getval": re.compile(
                r"""
                \s+df-election\spreempt-time\s(?P<preempt_time>\d+)
                $""", re.VERBOSE,
            ),
            "setval": "df-election preempt-time {{ df_election.preempt_time }}",
            "result": {
                "{{ segment }}": {
                    "df_election": {
                        "preempt_time": "{{ preempt_time }}",
                    },
                },
            },
        },
        {
            "name": "redundancy.all_active",
            "getval": re.compile(
                r"""
                \s+redundancy\sall-active
                $""", re.VERBOSE,
            ),
            "setval": "redundancy all-active",
            "result": {
                "{{ segment }}": {
                    "redundancy": {
                        "all_active": True,
                    },
                },
            },
        },
        {
            "name": "redundancy.single_active",
            "getval": re.compile(
                r"""
                \s+redundancy\ssingle-active
                $""", re.VERBOSE,
            ),
            "setval": "redundancy single-active",
            "result": {
                "{{ segment }}": {
                    "redundancy": {
                        "single_active": True,
                    },
                },
            },
        },
        {
            "name": "identifier",
            "getval": re.compile(
                r"""
                \s+identifier\stype\s(?P<identifier_type>0|3)
                (\s(?P<system_mac>system-mac))?
                (\s(?P<esi_value>.+))
                $""",
                re.VERBOSE,
            ),
            "setval": "identifier type {{ identifier.identifier_type }} "
            "{% if identifier.identifier_type == '3' %}system-mac "
            "{{ identifier.esi_value }}{% else %}{{ identifier.esi_value }}{% endif %}",
            "result": {
                "{{ segment }}": {
                    "identifier": {
                        "identifier_type": "{{ identifier_type }}",
                        "esi_value": "{{ esi_value }}",
                    },
                },
            },
        },
    ]
    # fmt: on



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
---
module: elb_application_lb_info
version_added: 5.0.0
short_description: Gather information about Application Load Balancers in AWS
description:
  - Gather information about Application Load Balancers in AWS.
  - This module was originally added to C(community.aws) in release 1.0.0.
author:
  - Rob White (@wimnat)
options:
  load_balancer_arns:
    description:
      - The Amazon Resource Names (ARN) of the load balancers. You can specify up to 20 load balancers in a single call.
    required: false
    type: list
    elements: str
  names:
    description:
      - The names of the load balancers.
    required: false
    type: list
    elements: str
  include_attributes:
    description:
      - Whether or not to include load balancer attributes in the response.
    required: false
    type: bool
    default: true
    version_added: 7.0.0
  include_listeners:
    description:
      - Whether or not to include load balancer listeners in the response.
    required: false
    type: bool
    default: true
    version_added: 7.0.0
  include_listener_rules:
    description:
      - Whether or not to include load balancer listener rules in the response.
      - Implies O(include_listeners=true)
    required: false
    type: bool
    default: true
    version_added: 7.0.0

extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
# Note: These examples do not set authentication details, see the AWS Guide for details.

- name: Gather information about all ALBs
  amazon.aws.elb_application_lb_info:

# Equivalent to aws elbv2 describe-load-balancers
- name: Gather minimal information about all ALBs
  amazon.aws.elb_application_lb_info:
    include_attributes: false
    include_listeners: false
    include_listener_rules: false

- name: Gather information about a particular ALB given its ARN
  amazon.aws.elb_application_lb_info:
    load_balancer_arns:
      - "arn:aws:elasticloadbalancing:ap-southeast-2:123456789012:loadbalancer/app/my-alb/aabbccddeeff"

- name: Gather information about ALBs named 'alb1' and 'alb2'
  amazon.aws.elb_application_lb_info:
    names:
      - alb1
      - alb2

- name: Gather information about specific ALB
  amazon.aws.elb_application_lb_info:
    names: "alb-name"
    region: "aws-region"
  register: alb_info
- ansible.builtin.debug:
    var: alb_info
"""

RETURN = r"""
load_balancers:
    description: A list of load balancers.
    returned: always
    type: complex
    contains:
        access_logs_s3_bucket:
            description: The name of the S3 bucket for the access logs.
            returned: when O(include_attributes=true)
            type: str
            sample: "mys3bucket"
        access_logs_s3_enabled:
            description: Indicates whether access logs stored in Amazon S3 are enabled.
            returned: when O(include_attributes=true)
            type: str
            sample: "true"
        access_logs_s3_prefix:
            description: The prefix for the location in the S3 bucket.
            returned: when O(include_attributes=true)
            type: str
            sample: "my/logs"
        availability_zones:
            description: The Availability Zones for the load balancer.
            type: list
            elements: dict
            sample: [{ "load_balancer_addresses": [], "subnet_id": "subnet-aabbccddff", "zone_name": "ap-southeast-2a" }]
            contains:
                load_balancer_addresses:
                    description: Information about static IP addresses for a load balancer.
                    type: list
                    elements: dict
                    contains:
                        ip_address:
                            description: The static IP address.
                            type: str
                        allocation_id:
                            description: The allocation ID of the Elastic IP address for an internal-facing load balancer.
                            type: str
                        private_ipv4_address:
                            description: The private IPv4 address for an internal load balancer.
                            type: str
                        ipv6_address:
                            description: The IPv6 address.
                            type: str
                subnet_id:
                    description: The ID of the subnet.
                    type: str
                zone_name:
                    description: The name of the Availability Zone.
                    type: str
        canonical_hosted_zone_id:
            description: The ID of the Amazon Route 53 hosted zone associated with the load balancer.
            type: str
            sample: "ABCDEF12345678"
        created_time:
            description: The date and time the load balancer was created.
            type: str
            sample: "2015-02-12T02:14:02+00:00"
        client_keep_alive_seconds:
            description: The client keep alive value, in seconds.
            returned: when O(include_attributes=true)
            type: str
            sample: "3600"
        connection_logs_s3_bucket:
            description: The name of the S3 bucket for the connection logs.
            returned: when O(include_attributes=true)
            type: str
            sample: ""
        connection_logs_s3_enabled:
            description: Indicates whether connection logs are enabled.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        connection_logs_s3_prefix:
            description: The prefix for the location in the S3 bucket for the connection logs.
            returned: when O(include_attributes=true)
            type: str
            sample: ""
        deletion_protection_enabled:
            description: Indicates whether deletion protection is enabled.
            returned: when O(include_attributes=true)
            type: str
            sample: "true"
        dns_name:
            description: The public DNS name of the load balancer.
            type: str
            sample: "internal-my-alb-123456789.ap-southeast-2.elb.amazonaws.com"
        idle_timeout_timeout_seconds:
            description: The idle timeout value, in seconds.
            returned: when O(include_attributes=true)
            type: str
            sample: "60"
        ip_address_type:
            description: The type of IP addresses used by the subnets for the load balancer.
            type: str
            sample: "ipv4"
        ipv6_deny_all_igw_traffic:
            description: Locks internet gateway (IGW) access to the load balancer.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        listeners:
            description: Information about the listeners.
            returned: when O(include_listeners=true) or O(include_listener_rules=true)
            type: complex
            contains:
                listener_arn:
                    description: The Amazon Resource Name (ARN) of the listener.
                    type: str
                    sample: ""
                load_balancer_arn:
                    description: The Amazon Resource Name (ARN) of the load balancer.
                    type: str
                    sample: ""
                port:
                    description: The port on which the load balancer is listening.
                    type: int
                    sample: 80
                protocol:
                    description: The protocol for connections from clients to the load balancer.
                    type: str
                    sample: "HTTPS"
                rules:
                    description: List of listener rules.
                    returned: when O(include_listener_rules=true)
                    type: list
                    elements: dict
                    contains:
                        rule_arn:
                            description: The Amazon Resource Name (ARN) of the rule.
                            type: str
                            sample: ""
                        priority:
                            description: The priority.
                            type: str
                            sample: "default"
                        is_default:
                            description: Indicates whether this is the default rule.
                            type: bool
                            sample: false
                        conditions:
                            description: The conditions.
                            type: list
                            sample: []
                        actions:
                            description: The actions.
                            type: list
                            elements: dict
                            contains:
                                type:
                                    description: The type of action.
                                    type: str
                                target_group_arn:
                                    description: The Amazon Resource Name (ARN) of the target group.
                                    type: str
                                forward_config:
                                    description: Information for creating an action that distributes requests among one or more target groups.
                                    type: dict
                                    contains:
                                        target_groups:
                                            description: The target groups.
                                            type: dict
                                            contains:
                                                target_group_arn:
                                                    description: The Amazon Resource Name (ARN) of the target group.
                                                    type: str
                                                weight:
                                                    description: The weight.
                                                    type: int
                                        target_group_stickiness_config:
                                            description: The target group stickiness for the rule.
                                            type: dict
                                            contains:
                                                enabled:
                                                    description: Indicates whether target group stickiness is enabled.
                                                    type: bool
                            sample: [
                                {
                                    "forward_config": {
                                        "target_group_stickiness_config": {
                                            "enabled": false
                                        },
                                        "target_groups": [
                                            {
                                                "target_group_arn": "arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/alb-test/09ba111f8079fb83",
                                                "weight": 1
                                            }
                                        ]
                                    },
                                    "target_group_arn": "arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/alb-test/09ba111f8079fb83",
                                    "type": "forward"
                                }
                            ]
                certificates:
                    description: The SSL server certificate.
                    type: list
                    elements: dict
                    contains:
                        certificate_arn:
                            description: The Amazon Resource Name (ARN) of the certificate.
                            type: str
                            sample: "arn:aws:acm:us-east-1:123456789012:certificate/28d2f3d9-cb2f-4033-a9aa-e75e704125a2"
                ssl_policy:
                    description: The security policy that defines which ciphers and protocols are supported.
                    type: str
                    sample: "ELBSecurityPolicy-2016-08"
                default_actions:
                    description: The default actions for the listener.
                    type: str
                    contains:
                        type:
                            description: The type of action.
                            type: str
                        target_group_arn:
                            description: The Amazon Resource Name (ARN) of the target group.
                            type: str
                        forward_config:
                            description: Information for creating an action that distributes requests among one or more target groups.
                            type: dict
                            contains:
                                target_groups:
                                    description: The target groups.
                                    type: dict
                                    contains:
                                        target_groups:
                                            description: The Amazon Resource Name (ARN) of the target group.
                                            type: str
                                        weight:
                                            description: The weight.
                                            type: int
                                target_group_stickiness_config:
                                    description: The target group stickiness for the rule.
                                    type: dict
                                    contains:
                                        enabled:
                                            description: Indicates whether target group stickiness is enabled.
                                            type: bool
                    sample: [
                        {
                            "forward_config": {
                                "target_group_stickiness_config": {
                                    "enabled": false
                                },
                                "target_groups": [
                                    {
                                        "target_group_arn": "arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/alb-test/bf43c68602c51c02",
                                        "weight": 1
                                    }
                                ]
                            },
                            "target_group_arn": "arn:aws:elasticloadbalancing:us-east-1:123456789012:targetgroup/alb-test/bf43c68602c51c02",
                            "type": "forward"
                        }
                    ]
        load_balancer_arn:
            description: The Amazon Resource Name (ARN) of the load balancer.
            type: str
            sample: "arn:aws:elasticloadbalancing:ap-southeast-2:123456789012:loadbalancer/app/my-alb/001122334455"
        load_balancer_name:
            description: The name of the load balancer.
            type: str
            sample: "my-alb"
        load_balancing_cross_zone_enabled:
            description: Indicates whether or not cross-zone load balancing is enabled.
            returned: when O(include_attributes=true)
            type: str
            sample: "true"
        routing_http2_enabled:
            description: Indicates whether HTTP/2 is enabled.
            returned: when O(include_attributes=true)
            type: str
            sample: "true"
        routing_http_desync_mitigation_mode:
            description: Determines how the load balancer handles requests that might pose a security risk to an application.
            returned: when O(include_attributes=true)
            type: str
            sample: "defensive"
        routing_http_drop_invalid_header_fields_enabled:
            description: Indicates whether HTTP headers with invalid header fields are removed by the load balancer (true) or routed to targets (false).
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        routing_http_preserve_host_header_enabled:
            description:
              - Indicates whether the Application Load Balancer should preserve the Host header in the HTTP request and send it to the target without any
                change.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        routing_http_x_amzn_tls_version_and_cipher_suite_enabled:
            description: Indicates whether the two headers are added to the client request before sending it to the target.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        routing_http_xff_client_port_enabled:
            description: Indicates whether the X-Forwarded-For header should preserve the source port that the client used to connect to the load balancer.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
        routing_http_xff_header_processing_mode:
            description:
              - Enables you to modify, preserve, or remove the X-Forwarded-For header in the HTTP request before the Application Load Balancer sends the
                request to the target.
            returned: when O(include_attributes=true)
            type: str
            sample: "append"
        scheme:
            description: Internet-facing or internal load balancer.
            type: str
            sample: "internal"
        security_groups:
            description: The IDs of the security groups for the load balancer.
            type: list
            sample: ["sg-0011223344"]
        state:
            description: The state of the load balancer.
            type: dict
            contains:
                code:
                    description: The state code.
                    type: str
                reason:
                    description: A description of the state.
                    returned: when available
                    type: str
            sample: {"code": "active"}
        tags:
            description: The tags attached to the load balancer.
            type: dict
            sample: {
                "Tag": "Example"
            }
        type:
            description: The type of load balancer.
            type: str
            sample: "application"
        vpc_id:
            description: The ID of the VPC for the load balancer.
            type: str
            sample: "vpc-0011223344"
        waf_fail_open_enabled:
            description: Indicates whether to allow a AWS WAF-enabled load balancer to route requests to targets
                if it is unable to forward the request to AWS WAF.
            returned: when O(include_attributes=true)
            type: str
            sample: "false"
"""

from typing import Dict

try:
    import botocore
except ImportError:
    pass  # Handled by AnsibleAWSModule

from ansible_collections.amazon.aws.plugins.module_utils.elb_utils import AnsibleELBv2Error
from ansible_collections.amazon.aws.plugins.module_utils.elb_utils import describe_load_balancers
from ansible_collections.amazon.aws.plugins.module_utils.elb_utils import describe_tags
from ansible_collections.amazon.aws.plugins.module_utils.elbv2 import build_application_load_balancer_description
from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.tagging import boto3_tag_list_to_ansible_dict


def get_load_balancer_tags(connection, load_balancer_arn: str) -> Dict[str, str]:
    tag_descriptions = describe_tags(connection, [load_balancer_arn])
    return boto3_tag_list_to_ansible_dict(tag_descriptions[0]["Tags"])


def list_load_balancers(connection, module: AnsibleAWSModule) -> None:
    load_balancer_arns = module.params.get("load_balancer_arns")
    names = module.params.get("names")
    include_attributes = module.params.get("include_attributes")
    include_listeners = module.params.get("include_listeners")
    include_listener_rules = module.params.get("include_listener_rules")

    try:
        load_balancers = describe_load_balancers(connection, load_balancer_arns=load_balancer_arns, names=names)

        # Build complete ALB descriptions with normalization
        snaked_load_balancers = [
            build_application_load_balancer_description(
                connection,
                lb,
                include_attributes=include_attributes,
                include_listeners=include_listeners,
                include_listener_rules=include_listener_rules,
            )
            for lb in load_balancers
        ]

        # Get tags for each load balancer (not included in the boto3 describe_load_balancers response)
        for snaked_load_balancer in snaked_load_balancers:
            snaked_load_balancer["tags"] = get_load_balancer_tags(connection, snaked_load_balancer["load_balancer_arn"])
    except AnsibleELBv2Error as e:
        module.fail_json_aws_error(e)

    module.exit_json(load_balancers=snaked_load_balancers)


def main():
    argument_spec = dict(
        load_balancer_arns=dict(type="list", elements="str"),
        names=dict(type="list", elements="str"),
        include_attributes=dict(default=True, type="bool"),
        include_listeners=dict(default=True, type="bool"),
        include_listener_rules=dict(default=True, type="bool"),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        mutually_exclusive=[["load_balancer_arns", "names"]],
        supports_check_mode=True,
    )

    try:
        connection = module.client("elbv2")
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e)

    list_load_balancers(connection, module)


if __name__ == "__main__":
    main()
